<?php

App::uses('Controller', 'Controller');
App::uses('Model', 'AppModel');
App::uses('ComponentCollection', 'Controller');
App::uses('XShell', 'Console/Command');
App::uses('Folder', 'Utility');
App::uses('File', 'Utility');
App::uses('CakeTime', 'Utility');

/**
 *
 * CreateFromFiles shell class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 7
 * @author Arnaud AUZOLAT
 * @copyright Initié par ADULLACT - Développé par Libriciel SCOP
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 *
 * @package		cake.app
 * @subpackage	cake.app.shell
 */
class ContactShell extends XShell {


	/**
	 *
	 * @var type
	 */
	public $files = array();


	/**
	 *
	 * @var type
	 */
	public $Collectivite;

	/**
	 *
	 * @var type
	 */
	public $Contact;

	/**
	 *
	 * @throws FatalErrorException
	 */
	public function startup() {
		parent::startup();

        $this->_conn = 'default';
		if (!empty($this->params['connection'])) {
			Configure::write('conn', $this->params['connection']);
			$this->_conn = $this->params['connection'];
		}
        
		$this->Collectivite = ClassRegistry::init('Collectivite');
		$this->Collectivite->useDbConfig = 'default';
		$coll = $this->Collectivite->find('first', array('conditions' => array('Collectivite.conn' => $this->params['connection'])));

                
		Configure::write('conn', $this->params['connection']);

        ClassRegistry::config(array( 'ds' => $this->params['connection']));
		$this->Contact = ClassRegistry::init('Contact');
        $this->Contact->useDbConfig = $this->_conn;

	}


	/**
	 *
	 */
	public function main() {

        if( !empty( $this->params['file']) /*&& !empty($this->params['addressbook'])*/ ){
            $createdContact = array();
            $this->out('<info>Acquisition des contacts en cours ...</info>');
            $this->XProgressBar->start(1);

            $this->importContactCsv($this->params['file'], $this->params['addressbook']);

            $this->hr();
            if (!in_array(false, $createdContact, true)) {
                $this->out('<success>Opération terminée avec succès.</success>');
            } else {
                $this->out('<error>Opération terminée avec erreur(s).</error>');
            }
            $this->hr();
        }
    }

	/**
	 *
	 * @return type
	 */
	public function getOptionParser() {
        $this->Contact = ClassRegistry::init('Contact');
         $addressbooksLists = $this->Contact->Addressbook->find('list', array('conditions' => array('Addressbook.active' => true), 'order' => array('Addressbook.name ASC')));

        $optionParser = parent::getOptionParser();
        $optionParser->description(__("Outils d'administration")); 
        $optionParser->addOption('connection', array(
			'short' => 'c',
			'help' => 'connection',
			'default' => 'default',
			'choices' => array_keys(ConnectionManager::enumConnectionObjects())
		));
        $optionParser->addOption('addressbook', array(
			'short' => 'a',
			'help' => 'addressbook',
			'choices' => array_keys($addressbooksLists)
		));
        $optionParser->addOption('file', array(
			'short' => 'f',
            'help' => 'Nom du fichier Contact (format CSV) à intégrer'
		));
        
                            
		return $optionParser;
	}
    
    
     
    /*
     * Fonction permettant d'intégrer le fichier BAN d'un département
     * 
     * 
     */

    private function _processCsv($contact) {
        
        $contacts = array(
            'modified' => $contact["DATE DE CREATION FICHE OU DERNIERE MODIFICATION"],
            'civilite' => $contact['CIVILITE'],
            'nom' => $contact['NOM'],
            'prenom' => $contact['PRENOM'],
            'fonction_id' => $contact['FONCTION'],
            'tel' => $contact['TELEPHONE'],
            'lignedirecte' => $contact['LIGNE DIRECTE'],
            'portable' => $contact['LIGNE MOBILE'],
            'email' => $contact['MAIL'],
            'adresse' => $contact['ADRESSE'],
            'compl' => $contact['COMPL ADRESSE'],
            'cp' => $contact['CODE POSTAL'],
            'ville' => $contact['VILLE'],
            'operationscontacts' => $contact['OPERATIONS DU CONTACT'],
            'observations' => $contact['OBSERVATIONS'],
            'active' => 1
        );
        
        $orgs = array(
            'name' => $contact['ORGANISME'],
            'antenne' => $contact['ANTENNE'],
            'activites' => $contact['ACTIVITE'],
            'operationsorgs' => $contact["OPERATIONS DE L'ORGANISME"],
            'siret' => $contact["SIRET"],
            'fax' => $contact["FAX"]
        );
        
//        $events = array(
//            'events' => $contact[]
//        );
    

        $res = array(
            'contacts' => $contacts,
            'orgs' => $orgs
        );
        return $res;
    }

    
    /**
     *
     * @param type $filename
     * @param type $contact_id
     * @param type $foreign_key
     * @return boolean
     */
    public function importContactCsv($filename = null, $addressbookId = null, $foreign_key = null) {
        $this->Contact->useDbConfig = $this->_conn;
        
//ini_set( 'memory_limit', '3000M');
        $return = array();
        if ($filename != null && is_file($filename) && is_readable($filename)) {
            if (($handle = fopen($filename, 'r')) !== FALSE) {
                
                $contacts = array();
                $header = NULL;

                while (($row = fgetcsv($handle, 1000000, ',')) !== FALSE) {
                    if (!$header) {
                        $header = $row;
                    } else {
                        $tmp = array();
                        for ($i = 0; $i < count($header); $i++) {
                            $tmp[$header[$i]] = $row[$i];
                        }
                        $contacts[] = $tmp;
                    }
                }
                fclose($handle);
                


                $this->XProgressBar->start(count($contacts)); 
                $this->out('<info>Enregistrement des contacts...</info>');
                foreach ($contacts as $contact) {

                    $contactINFO = $this->_processCsv($contact);


                    // On sauvegarde d'abord l'organisme
                    if( !empty( $contactINFO['orgs'] ) ){
                        $orgINFO['Organisme'] = $contactINFO['orgs'];
                        $orgINFO['Organisme']['addressbook_id'] = $addressbookId;
                        $organisme = $this->Contact->Organisme->find(
                            'first',
                            array(
                                'conditions' => array(
                                    'Organisme.name' => $orgINFO['Organisme']['name']
                                ),
                                'recursive' => -1
                            )
                        );
                        // Si l'organisme existe en BDD, on note son ID
                        if( !empty($organisme) ) {
                            $OrgId = $organisme['Organisme']['id'];
                        }
                        // sinon, on crée une entrée en BDD
                        else {
                            $this->Contact->Organisme->create();
                            $this->Contact->Organisme->save($orgINFO);
                            $OrgId = $this->Contact->Organisme->id;
                        }
                    }
                    
                    // ensuite on passe aux fonctions du contact (1 par contact)
                    if( !empty( $contactINFO['contacts']['fonction_id'] ) ){
                        $fonctionINFO = array();
                        $fonctionINFO['Fonction']['name'] = $contactINFO['contacts']['fonction_id'];
                        $fonction = $this->Contact->Fonction->find(
                            'first',
                            array(
                                'conditions' => array(
                                    'Fonction.name' => $fonctionINFO['Fonction']['name']
                                ),
                                'recursive' => -1
                            )
                        );
                        // Si la fonction existe en BDD, on note son ID
                        if( !empty($fonction) ) {
                            $FonctionId = $fonction['Fonction']['id'];
                        }
                        else {
                            // sinon on crée l'entrée en BDD
                            $this->Contact->Fonction->create();
                            $this->Contact->Fonction->save($fonctionINFO);
                            $FonctionId = $this->Contact->Fonction->id;
                        }
                        $contactINFO['contacts']['fonction_id'] = $FonctionId;
                    }
                    
                    // gestion des événements
                    if( !empty($contactINFO['events']) ) {
                        
                    }
                    
                    // ensuite on sauvegarde le contact
                    if( !empty($contactINFO['contacts']) ) {
                        $this->Contact->begin();
                        $contactValue = $contactINFO['contacts'];
                        $contactAlreadyExist = $this->Contact->find(
                            'first',
                            array(
                                'conditions' => array(
                                    'Contact.name' => $contactValue['prenom'].' '.$contactValue['nom'],
                                    'Contact.nom' => $contactValue['nom'],
                                    'Contact.prenom' => $contactValue['prenom'],
                                    'Contact.civilite' => $contactValue['civilite']
                                ),
                                'recursive' => -1
                            )
                        );
//debug($contactAlreadyExist);
                        if( empty( $contactAlreadyExist ) ) {
                            $this->Contact->create();
                            $contactDATA['Contact'] = $contactINFO['contacts'];
                            $contactDATA['Contact']['name'] = $contactINFO['contacts']['prenom'].' '.$contactINFO['contacts']['nom'];
                            $contactDATA['Contact']['organisme_id'] = $OrgId;
    //debug($contactDATA['Contact']);
    //die();
                            $contactDATA['Contact']['addressbook_id'] = $addressbookId;

                            $modified = str_replace( ' à ', ' ', $contactINFO['contacts']['modified']);
                            $modified = date('Y-d-m H:i:s', strtotime($modified));

                            $contactDATA['Contact']['modified'] = $modified;
                            if( !empty( $contactINFO['contacts']['fonction_id'] ) ){
                                $contactDATA['Contact']['fonction_id'] = $FonctionId;
                            }
                            $saved = $this->Contact->save($contactDATA);
                            $contactId = $this->Contact->id;
                        }
                        else {
                            $contactId = $contactAlreadyExist['Contact']['id'];
                            $saved = true;
                        }
                        
                        

                        // On récupère les colonnes supplémentaires et on vérifie si les events existent ou non
                        foreach( $header as $h => $value ) {
                            if($h > 20) {
                                $event = $this->Contact->Event->find(
                                    'first',
                                    array(
                                        'conditions' => array(
                                            'Event.name ILIKE' => $value
                                        ),
                                        'recursive' => -1
                                    )
                                );

                                // Si la fonction existe en BDD, on note son ID
                                if( !empty($event) ) {
                                    $EventId = $event['Event']['id'];
                                }
                                else {
                                    // sinon on crée l'entrée en BDD
                                    $eventINFO['Event']['name'] = ucwords(strtolower($value));
                                    $this->Contact->Event->create();
                                    $this->Contact->Event->save($eventINFO);
                                    $EventId = $this->Contact->Event->id;
                                }

                                // On récupère les événements du CSV
                                $contactINFO['contacts']['events'][$value] = $contact[$value];
                                
                                
                                foreach($contactINFO['contacts']['events'] as $nameEvent => $boolean) {
                                    // On ne prend en compte que ceux qui sont à Oui
                                    if( $boolean == 'Oui' ) {
                                        $eventContact = $this->Contact->Event->find(
                                            'first',
                                            array(
                                                'conditions' => array(
                                                    'Event.name ILIKE' => $nameEvent
                                                ),
                                                'recursive' => -1
                                            )
                                        );
                                        $linkContactEventsExists = $this->Contact->ContactEvent->find(
                                            'all',
                                            array(
                                                'conditions'  => array(
                                                    'ContactEvent.contact_id' => $contactId,
                                                    'ContactEvent.event_id' => $eventContact['Event']['id']
                                                ),
                                                'contain' => false
                                            )
                                        );
                                        // si aucune entrée, alors on enregistre
                                        if( empty($linkContactEventsExists) ) {
                                            $liaisonEventContact['ContactEvent']['event_id'] = $eventContact['Event']['id'];
                                            $liaisonEventContact['ContactEvent']['contact_id'] = $contactId;
                                            $this->Contact->ContactEvent->create();
                                            $this->Contact->ContactEvent->saveAll($liaisonEventContact);
                                        }
                                    }
                                }
                            }

                        }
                        // on récupère les valeurs des événements et on les stocke par contact

//debug($contactINFO['contacts']);
                        
                        // ensuite on sauvegarde les relations entre contacts et OP
                        if($saved) {
                            $contactops = array();
                            $contactops = explode( ' - ', $contactINFO['contacts']['operationscontacts']);
                            // On récupère la liste des OP associés au contact
                            $operationsContacts = $this->Contact->Operation->find(
                                'all',
                                array(
                                    'conditions' => array(
                                        'Operation.name' => $contactops
                                    ),
                                    'recursive' => -1
                                )
                            );

                            // On vérifie si une OP existe
                            if( !empty($operationsContacts) ) {
                                $liaisonContact = array();
                                // pour chacune des OP
                                foreach( $operationsContacts as $op => $operation ) {
                                    $operationId = $operation['Operation']['id'];
                                    

                                    // on regarde si une entrée existe dnas la table de liasion entre les contacts et les OP
                                    $linkContactOPExists = $this->Contact->ContactOperation->find(
                                        'all',
                                        array(
                                            'conditions'  => array(
                                                'ContactOperation.contact_id' => $contactId,
                                                'ContactOperation.operation_id' => $operationId
                                            ),
                                            'contain' => false
                                        )
                                    );
//debug($operationId);
//debug($linkContactOPExists);

                                    // si aucune entrée, alors on enregistre
                                    if( empty($linkContactOPExists) ) {
                                        $liaisonContact['ContactOperation']['operation_id'] = $operationId;
                                        $liaisonContact['ContactOperation']['contact_id'] = $contactId;
                                        $this->Contact->ContactOperation->create();
                                        $this->Contact->ContactOperation->saveAll($liaisonContact);
                                    }
                                    // sinon on ne fait rien
                                }
                            }
                            else {
                                // si l'OP n'existe pas, on la crée
                                foreach($contactops as $opname) {
                                    $dataOP['Operation']['name'] = $opname;
                                    $this->Contact->Operation->create();
                                    $this->Contact->Operation->save($dataOP);
                                }
                            }
                                                        
                            // ensuite on sauvegarde les relations entre organismes et OP
                            $orgops = array();
                            $orgops = explode( ' - ', $contactINFO['orgs']['operationsorgs']);
                            $operationsorgs = $this->Contact->Organisme->Operation->find(
                                'all',
                                array(
                                    'conditions' => array(
                                        'Operation.name' => $orgops
                                    ),
                                    'recursive' => -1
                                )
                            );
                            if( !empty($operationsorgs) ) {
                                $liaisonOrg = array();
                                // pour chacune des OP
                                foreach( $operationsorgs as $orgop => $operationorg ) {
                                    $operationOrgId = $operationorg['Operation']['id'];
                                    // on regarde si une entrée existe dnas la table de liasion entre les contacts et les OP
                                    $linkOrgOPExists = $this->Contact->Organisme->OrganismeOperation->find(
                                        'all',
                                        array(
                                            'conditions'  => array(
                                                'OrganismeOperation.organisme_id' => $OrgId,
                                                'OrganismeOperation.operation_id' => $operationOrgId
                                            ),
                                            'contain' => false
                                        )
                                    );
                                    // si aucune entrée, alors on enregistre
                                    if( empty($linkOrgOPExists) ) {
                                        $liaisonOrg['OrganismeOperation']['operation_id'] = $operationOrgId;
                                        $liaisonOrg['OrganismeOperation']['organisme_id'] = $OrgId;
                                        $this->Contact->Organisme->OrganismeOperation->create();
                                        $this->Contact->Organisme->OrganismeOperation->saveAll($liaisonOrg);
                                    }    
                                    // sinon on ne fait rien
                                }
                            }
                            else {
                                // si l'OP n'existe pas, on la crée
                                foreach($orgops as $oporgname) {
                                    $dataOrgOP['Operation']['name'] = $oporgname;
                                    $this->Contact->Organisme->Operation->create();
                                    $this->Contact->Organisme->Operation->save($dataOrgOP);
                                }
                            }
                            
                            // pour les activités
                            $orgactivites = explode( ' - ', $contactINFO['orgs']['activites']);
                            foreach($orgactivites as $activiteorgname) {
                                $dataOrgActivite['Activite']['name'] = $activiteorgname;
                                $activiteAlreadySet = $this->Contact->Organisme->Activite->find(
                                    'first',
                                    array(
                                        'conditions' => array(
                                            'Activite.name' => $activiteorgname
                                        ),
                                        'recursive' => -1
                                    )
                                );
                                if( empty($activiteAlreadySet) ) {
                                    $this->Contact->Organisme->Activite->create();
                                    $this->Contact->Organisme->Activite->save($dataOrgActivite);
                                }
                            }
//debug($orgactivites);
//debug($contactINFO['orgs']['activites']);
                            $activitesorgs = $this->Contact->Organisme->Activite->find(
                                'all',
                                array(
                                    'conditions' => array(
                                        'Activite.name' => $orgactivites
                                    ),
                                    'recursive' => -1
                                )
                            );

                            if( !empty($activitesorgs) ) {
                                $liaisonActivite = array();
                                // pour chacune des OP
                                foreach( $activitesorgs as $actorg => $activite ) {
                                    $activiteOrgId = $activite['Activite']['id'];
                                    // on regarde si une entrée existe dnas la table de liasion entre les contacts et les OP
                                    $linkOrgActiviteExists = $this->Contact->Organisme->ActiviteOrganisme->find(
                                        'all',
                                        array(
                                            'conditions'  => array(
                                                'ActiviteOrganisme.organisme_id' => $OrgId,
                                                'ActiviteOrganisme.activite_id' => $activiteOrgId
                                            ),
                                            'contain' => false
                                        )
                                    );
//debug($linkOrgActiviteExists);
                                    // si aucune entrée, alors on enregistre
                                    if( empty($linkOrgActiviteExists) ) {

                                        $liaisonActivite['ActiviteOrganisme']['activite_id'] = $activiteOrgId;
                                        $liaisonActivite['ActiviteOrganisme']['organisme_id'] = $OrgId;
                                        $this->Contact->Organisme->ActiviteOrganisme->create();
                                        $this->Contact->Organisme->ActiviteOrganisme->saveAll($liaisonActivite);
                                    }    
                                    // sinon on ne fait rien
                                }
                            }
                        }
                    }

                    if ($saved) {
                        $return[] = true;
                        $this->Contact->commit();

                    } else {
                        $return[] = false;
                        $this->Contact->rollback();
                    }
                    $this->XProgressBar->next();
                }
                $this->XProgressBar->finish();
            }
        }
        return !in_array(false, $return, true);
    }
    
    
}