<?php

App::uses('ShellDispatcher', 'Console');

/**
 * Collectivités
 *
 * Collectivites controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 7
 * @author Stéphane Sampaio
 * @copyright Initié par ADULLACT - Développé par ADULLACT Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		Controller
 */
class CollectivitesController extends AppController {

    /**
     *
     * Controller name
     *
     * @var array
     */
    public $name = 'Collectivites';

    /**
     *
     * Controller uses
     *
     * @var array
     */
    public $uses = array('Collectivite');

    /**
     * Fonction de rappel beforeFilter
     *
     * @description  Paramétrage spécifique du composant Auth. Si on est connecté en tant que superadmin, on peut accèder à toutes les pages sans utiliser les Acl de CakePHP.
     *
     * @access public
     * @return void
     */
    public function beforeFilter() {
        parent::beforeFilter();
        if ($this->Auth->User('id') && Configure::read('conn') == "default") {
            $this->Auth->allow('*');
        }
    }

    /**
     * Gestion des collectivités (interface graphique)
     *
     * @logical-group Collectivité
     * @user-profile Superadmin
     *
     * @access public
     * @return void
     */
    public function index() {
        $this->set('ariane', array(
            '<a href="/environnement">' . __d('menu', 'Administration') . '</a>',
            __d('menu', 'gestionCollectivites')));
    }

    /**
     * Récupération de la liste des collectivités (ajax)
     *
     * @logical-group Collectivité
     * @user-profile Superadmin
     *
     * @access public
     * @return void
     */
    public function getCollectivites() {
        $collectivites_tmp = $this->Collectivite->find("all", array('order' => 'Collectivite.name'));
        $collectivites = array();
        foreach ($collectivites_tmp as $item) {
            $item['right_edit'] = true;
            $item['right_delete'] = $this->Collectivite->isDeletable($item['Collectivite']['id']);
            $collectivites[] = $item;
        }
        $this->set('collectivites', $collectivites);
    }

    /**
     *
     * @param type $id *
     */
    //TODO: supprimer les fonctions commentées
//	public function view($id) {
//		$collectivite = $this->Collectivite->find(
//				'first', array(
//			'contain' => array(
//				'Config',
//				'Service',
//			),
//			'conditions' => array('Collectivite.id' => $id)
//				)
//		);
//		if (empty($collectivite)) {
//			throw new NotFoundException();
//		}
//		$this->set(compact('collectivite'));
//	}

    /**
     * Validation d'unicité du nom d'une collectivité (création / édition)
     *
     * @param type $field
     */
    public function ajaxformvalid($field = null) {
        $models = array();
        if ($field != null) {
            $models['Collectivite'] = array($field);
        } else {
            $models[] = 'Collectivite';
        }
        $this->Ajaxformvalid->valid($models, $this->request->data);
    }

    /**
     * Récupération de la liste des connexions disponible via le fichier APP/Config/database.php
     *
     * @logical-group Collectivité
     * @logical-group Ajout d'une collectivité
     *
     * @logical-used-by Collectivités > Ajout d'une collectivité
     * @user-profile Superadmin
     *
     * @access public
     * @return void
     */
    public function getConnList() {
        $conns = ConnectionManager::enumConnectionObjects();
        unset($conns['default']);

        //suppression des connexion dèjà paramétrées pour une collectivite
        $listCollectiviteConn = $this->Collectivite->find('list', array('fields' => array('Collectivite.id', 'Collectivite.conn')));
        foreach ($listCollectiviteConn as $key => $val) {
            unset($conns[$val]);
        }

        $connsToSend = array();
        foreach (array_keys($conns) as $conn) {
            $connsToSend[$conn] = $conn;
        }

        $this->Jsonmsg->sendJson($connsToSend);
    }

    /**
     * Ajout d'une collectivité
     * 	- informations
     * 	- paramétrages
     *  - administrateur de la collectivité
     *  - création des tables dans la base de données
     *  - ajout des données de base
     *  - création des séquences / compteurs pour les dossiers et affaires
     *
     * @logical-group Collectivité
     * @user-profile Superadmin
     *
     * @access public
     * @return void
     */
    public function add() {
        //recuperation de la liste des connexions existantes dans le fichier database.php
        $conns = ConnectionManager::enumConnectionObjects();
        if (!empty($this->request->data)) {
            $valid = array(
                'collectivite' => false,
                'import' => false,
                'admin' => false,
                'cpt' => false,
                'success' => false,
                'msg' => __d('collectivite', 'Collectivite.add.error'),
                'class' => 'alert-danger'
            );

// 			$this->Collectivite->begin();
            $this->Jsonmsg->init();


            //creation de la collectivite dans la base admin
            $this->Collectivite->create();

            try {
                // Création de la BDD servant de "modèle" pour toutes les collectivités
                $this->Collectivite->query('CREATE DATABASE ' . $conns['template']['database'] . ' OWNER ' . $conns['template']['login'] . ';');
                // Insertion du fichier SQL contenant les infos de la BDD "modèle"
                $filename = APP . 'Config/sql/webgfc_template.sql';
                $cmdCreate = 'psql -U ' . $conns['template']['login'] . ' -p ' . $conns['template']['port'] . ' -h ' . $conns['template']['host'] . ' -f ' . $filename . ' ' . $conns['template']['database'];
                putenv('PGPASSWORD=' . $conns['template']['password']);
                exec(escapeshellcmd($cmdCreate), $output, $return_var);
                putenv('PGPASSWORD=""');

                // Création de la BDD collectivité s'appuyant sur le "modèle" préalablement créé
                $this->Collectivite->query('CREATE DATABASE ' . $conns[$this->request->data['Collectivite']['conn']]['database'] . ' TEMPLATE ' . $conns['template']['database'] . ';');
                $this->Collectivite->begin();

                //on force la collectivité à être active par défaut
                $this->request->data['Collectivite']['active'] = true;

                if ($this->Collectivite->save($this->request->data)) {
                    $valid['collectivite'] = true;

// 					$this->request->data[''][''];
                    // Création BDD vide
                    // 				$bddCreate = 'psql -U ' . $conns[$this->request->data['Collectivite']['conn']]['login'] . ' -p ';
                    // 				$bddCreate = 'su - postgres -c "psql -c \"CREATE DATABASE $1 WITH OWNER webgfc\"';
                    //import du squelette
                    // 				$filename = APP . 'Config/sql/100_new_conn.sql';
                    // 				$cmdCreate = 'psql -U ' . $conns[$this->request->data['Collectivite']['conn']]['login'] . ' -p ' . $conns[$this->request->data['Collectivite']['conn']]['port'] . ' -h ' . $conns[$this->request->data['Collectivite']['conn']]['host'] . ' -f ' . $filename . ' ' . $conns[$this->request->data['Collectivite']['conn']]['database'];
                    // 				$filename = APP . 'Config/sql/101_minimal_data_set.sql';
                    // 				$cmdInit = 'psql -U ' . $conns[$this->request->data['Collectivite']['conn']]['login'] . ' -p ' . $conns[$this->request->data['Collectivite']['conn']]['port'] . ' -h ' . $conns[$this->request->data['Collectivite']['conn']]['host'] . ' -f ' . $filename . ' ' . $conns[$this->request->data['Collectivite']['conn']]['database'];
                    // 
                    // 				putenv('PGPASSWORD=' . $conns[$this->request->data['Collectivite']['conn']]['password']);
                    // 				exec(escapeshellcmd($cmdCreate), $output, $return_var);
                    // 				if ($return_var == 0) {
                    // 					exec(escapeshellcmd($cmdInit), $output, $return_var);
                    // 				}
                    // 				putenv('PGPASSWORD=""');
                    // 				if ($return_var == 0) {
                    $valid['import'] = true;

                    //creation de l'administrateur dans la base collectivite
                    $this->loadModel('Desktop');
                    $this->Desktop->setDataSource($this->request->data['Collectivite']['conn']);
                    $this->loadModel('User');
                    $this->User->setDataSource($this->request->data['Collectivite']['conn']);
                    $this->loadModel('Aco');
                    $this->Aco->setDataSource($this->request->data['Collectivite']['conn']);
                    $this->loadModel('Aro');
                    $this->Aro->setDataSource($this->request->data['Collectivite']['conn']);
                    $this->loadModel('Permission');
                    $this->Permission->setDataSource($this->request->data['Collectivite']['conn']);
                    $this->User->begin();

                    $desktop_admin = $this->Desktop->create();
                    $desktop_admin['Desktop']['name'] = 'Rôle administrateur';
                    $desktop_admin['Desktop']['group_id'] = ADMIN_GID;
                    if ($this->Desktop->save($desktop_admin)) {
                        $user = array('User' => $this->request->data['User']);
                        $user['User']['desktop_id'] = $this->Desktop->id;
                        $this->User->create($user);
                        if ($this->User->save()) {
                            $valid['admin'] = true;
                            $valid['success'] = true;
                        }
                    }

                    //creation des compteurs dossiers / affaires dans la base collectivite
                    $this->loadModel('Compteur');
                    $this->Compteur->useDbConfig = $this->request->data['Collectivite']['conn'];
                    $this->loadModel('Sequence');
                    $this->Sequence->useDbConfig = $this->request->data['Collectivite']['conn'];
                    //sequence affaire
                    $sequence_affaire = $this->Sequence->create();
                    $sequence_affaire['Sequence']['nom'] = 'sequence_affaire';
                    $sequence_affaire['Sequence']['commentaire'] = 'sequence_affaire';
                    $sequence_affaire['Sequence']['num_sequence'] = 1;
                    if ($this->Sequence->save($sequence_affaire)) {
                        //compteur affaire
                        $compteur_affaire = $this->Compteur->create();
                        $compteur_affaire['Compteur']['nom'] = 'compteur_affaire';
                        $compteur_affaire['Compteur']['commentaire'] = 'compteur_affaire';
                        $compteur_affaire['Compteur']['def_compteur'] = "AFF_#AAAA##S#";
                        $compteur_affaire['Compteur']['def_reinit'] = "#AAAA#";
                        $compteur_affaire['Compteur']['sequence_id'] = $this->Sequence->id;
                        if ($this->Compteur->save($compteur_affaire)) {
                            //collectivite compteur affaire
                            //sequence dossier
                            $sequence_dossier = $this->Sequence->create();
                            $sequence_dossier['Sequence']['nom'] = 'sequence_dossier';
                            $sequence_dossier['Sequence']['commentaire'] = 'sequence_dossier';
                            $sequence_dossier['Sequence']['num_sequence'] = 1;
                            if ($this->Sequence->save($sequence_dossier)) {
                                //compteur dossier
                                $compteur_dossier = $this->Compteur->create();
                                $compteur_dossier['Compteur']['nom'] = 'compteur_dossier';
                                $compteur_dossier['Compteur']['commentaire'] = 'compteur_dossier';
                                $compteur_dossier['Compteur']['def_compteur'] = "DOS_#AAAA##S#";
                                $compteur_dossier['Compteur']['def_reinit'] = "#AAAA#";
                                $compteur_dossier['Compteur']['sequence_id'] = $this->Sequence->id;
                                if ($this->Compteur->save($compteur_dossier)) {
                                    //collectivite compteur dossier
                                    $valid['cpt'] = true;
                                }
                            }
                        }
                    }
                    // 				}
                }
            } catch (Exception $exc) {
                $valid['msg'] .= '<br /><br />' . __d('collectivite', 'Collectivite.add.error.db') . '<hr />' . (Configure::read('debug') > 0 ? $exc->getTraceAsString() : '');
            }

            if (!in_array(false, $valid, true)) {
//        $this->_initAcl($this->request->data['Collectivite']['conn']);
                $this->Jsonmsg->valid();
                $this->Collectivite->commit();
                if ($valid['import'] == true) {
                    $this->User->commit();
                }
                // Destruction de la BDD modèle pour les prochaines colelctivités
                $this->Collectivite->query('DROP DATABASE ' . $conns['template']['database'] . ';');
            } else {
                $this->Jsonmsg->error(__d('collectivite', 'Collectivite.add.error'));
                $this->Jsonmsg->debug(var_export($valid, true));
                $this->Collectivite->rollback();
                if ($valid['import'] == true) {
                    $this->User->rollback();
                }
            }
            $this->Jsonmsg->send();
        } else {
            //suppression de la base admin (default)
            unset($conns['default']);
            //suppression de la base stats (uniquement en dev)
            if (isset($conns['stats'])) {
                unset($conns['stats']);
            }

            //suppression des connexion dèjà paramétrées pour une collectivite
            $listCollectiviteConn = $this->Collectivite->find('list', array('fields' => array('Collectivite.id', 'Collectivite.conn')));
            foreach ($listCollectiviteConn as $key => $val) {
                unset($conns[$val]);
            }

            $connsToSend = array();
            foreach (array_keys($conns) as $conn) {
                $connsToSend[$conn] = $conn;
            }
            $this->set('conns', $connsToSend);
        }
    }

    /**
     * Edition d'une collectivité
     *
     * @logical-group Collectivité
     * @user-profile Superadmin
     *
     * @access public
     * @param integer $id
     * @return void
     */
    public function edit($id) {
        if (!empty($this->request->data)) {
            $this->Jsonmsg->Init();
            $conn = $this->Collectivite->field('conn', array('Collectivite.id' => $this->request->data['Collectivite']['id']));
            if (!empty($conn)) {

                $valid = array(
                    'collectivite' => false,
                    'user' => false,
//					'compteur_affaire' => false,
//					'compteur_dossier' => false
                );

                $this->Collectivite->begin();
                $this->Collectivite->create($this->request->data);
                if ($this->Collectivite->save()) {
                    $valid['collectivite'] = true;
                }

                $this->loadModel('User');
                $this->User->setDataSource($conn);
                $this->User->begin();

                if (!empty($this->request->data['User']['Newpassword'])) {
                    $this->request->data['User']['password'] = $this->request->data['User']['Newpassword'];
                }

                $this->User->create(array('User' => $this->request->data['User']));
                if ($this->User->save()) {
                    $valid['user'] = true;
                }

                if (!in_array(false, $valid, true)) {
                    $this->Jsonmsg->valid();
                    $this->Collectivite->commit();
                    $this->User->commit();
                } else {
                    $this->Jsonmsg->error(var_export($valid, true));
                    $this->Collectivite->rollback();
                    $this->User->rollback();
                }
                $this->Jsonmsg->send();
            } else {
                throw new NotFoundException();
            }
        } else {
            $this->request->data = $this->Collectivite->find('first', array('conditions' => array('Collectivite.id' => $id)));

            $conn = $this->request->data['Collectivite']['conn'];

            $this->loadModel('User');
            $this->User->setDataSource($conn);
            $this->loadModel('Desktop');
            $this->Desktop->setDataSource($conn);


            $desktopId = $this->Desktop->field('id', array('Desktop.group_id' => ADMIN_GID));
            $admin = $this->User->find('first', array('conditions' => array('User.desktop_id' => $desktopId)));
            unset($admin['User']['password']);
            $this->request->data['User'] = $admin['User'];

            // Gestion par bureau
            $this->loadModel('Desktopmanager');
            $this->Desktopmanager->setDataSource($conn);
            $optionsScanDesktopId = $this->Desktopmanager->find('list', array('conditions' => array('Desktopmanager.isdispatch' => true)));
            $this->set('optionsScanDesktopId', $optionsScanDesktopId);

            if (empty($this->request->data)) {
                throw new NotFoundException();
            }
        }
    }

    /**
     * Suppression d'une collectivité (soft delete)
     *
     * @logical-group Collectivité
     * @user-profile Superadmin
     *
     * @access public
     * @param integer $id
     * @return void
     */
    public function delete($id = null) {
        $this->Jsonmsg->init(__d('default', 'delete.error'));
        $this->Collectivite->begin();
        if ($this->Collectivite->delete($id, true)) {
            $this->Collectivite->commit();
            $this->Jsonmsg->valid(__d('default', 'delete.ok'));
        } else {
            $this->Collectivite->rollback();
        }
        $this->Jsonmsg->send();
    }

    /**
     *
     * @param type $id
     */
    //TODO: supprimer les fonctions commentées
//	public function fillOptionList($id = null) {
//		$this->set('collectiviteOptionList', $this->Collectivite->find('list', array('order' => 'Collectivite.name')));
//		if ($id != null) {
//			$this->set('id', $id);
//		}
//	}

    /**
     * Envoyer le logo d'une collectivité
     *
     * @logical-group Collectivité
     * @logical-group Edition d'une collectivité
     *
     * @logical-used-by Collectivité > Edition d'une collectivité
     * @user-profile Superadmin
     * @user-profile Admin
     *
     * @access public
     * @return void
     */
    public function upload() {
        if (!empty($_FILES['myfile'])) {
            $filename = basename(strtr($_FILES['myfile']['name'], " ", "_"));
            $target = LOGO_DIR . $filename;
            $message = '';
            if (@move_uploaded_file($_FILES['myfile']['tmp_name'], $target)) {
                $message = 'Le fichier a été envoyé.';
                //enregistrer le lienlogo dans la collectivite
                $this->Collectivite->id = $_POST['collectiviteId'];
                //			$collectivite = $this->Collectivite->read();
                $collectivite = $this->Collectivite->find(
                        'first', array(
                    'fields' => array(
                        'Collectivite.id',
                        'Collectivite.logo'
                    ),
                    'conditions' => array(
                        'Collectivite.id' => $this->Collectivite->id
                    ),
                    'contain' => false
                        )
                );
                //debug($collectivite);
                //die();
                $collectivite['Collectivite']['logo'] = $filename;
                $this->Collectivite->save($collectivite);
            } else {
                $message = 'Le fichier n\'a pas été envoyé';
            }
            //affichage du message d erreur
            //on utilisera ici jgrowl directement dans la vue, etant donne que l upload se fait via une iframe et que l utilisation
            //classique de jgrowl afficherai le message dans cette iframe invisible
            $this->set('message', $message);
            sleep(1);
        }
    }

    /**
     * Récupération du logo d'une collectivité
     *
     * @logical-group Collectivité
     *
     * @logical-used-by Collectivité > Edition d'une collectivité
     * @user-profile Superadmin
     * @user-profile Admin
     *
     * @logical-used-by Environnement > Environnement administrateur
     * @user-profile Admin
     *
     * @access public
     * @param integer $collectiviteId
     * @return void
     */
    public function getLogo($collectiviteId = null) {
        if ($collectiviteId != null) {
            $this->Collectivite->id = $collectiviteId;
//			$collectivite = $this->Collectivite->read();
            $collectivite = $this->Collectivite->find(
                    'first', array(
                'fields' => array(
                    'Collectivite.id',
                    'Collectivite.logo'
                ),
                'conditions' => array(
                    'Collectivite.id' => $collectiviteId
                ),
                'contain' => false
                    )
            );

            $this->set('col_id', $collectiviteId);
            $this->set('logo_url', $collectivite['Collectivite']['logo']);
        }
    }

    /**
     * Edition d'une collectivité
     *
     * @logical-group Collectivité
     * @user-profile Admin
     *
     * @access public
     * @return void
     */
    public function admin() {
        $this->set('ariane', array(
            '<a href="/environnement/index/0/admin">' . __d('menu', 'Administration', true) . '</a>',
            __d('default', 'Informations', true)
        ));
        $conn = $this->Session->read('Auth.User.connName');

        if (!empty($this->request->data)) {
            $valid = array(
                'collectivite' => false,
                'compteur_dossier' => false,
                'compteur_affaire' => false
            );
            $this->Jsonmsg->init();
            $this->Collectivite->begin();
            $this->loadModel('Compteur');
            $this->Compteur->useDbConfig = $conn;
            $this->Compteur->begin();

            $this->Collectivite->create($this->request->data);
            if ($this->Collectivite->save()) {
                $valid['collectivite'] = true;
            }


            $this->request->data['Caffaire']['id'] = $this->Compteur->field('Compteur.id', array('Compteur.nom' => 'compteur_affaire'));
            $this->Compteur->create(array('Compteur' => $this->request->data['Caffaire']));
            if ($this->Compteur->save()) {
                $valid['compteur_affaire'] = true;
            }
            $this->request->data['Cdossier']['id'] = $this->Compteur->field('Compteur.id', array('Compteur.nom' => 'compteur_dossier'));
            $this->Compteur->create(array('Compteur' => $this->request->data['Cdossier']));
            if ($this->Compteur->save()) {
                $valid['compteur_dossier'] = true;
            }


            if (!in_array(false, $valid, true)) {
                $this->Jsonmsg->valid();
                $this->Collectivite->commit();
                $this->Compteur->commit();
            } else {
                $this->Collectivite->rollback();
                $this->Compteur->rollback();
            }
            $this->Jsonmsg->send();
        } else {
            $this->request->data = $this->Collectivite->find('first', array('conditions' => array('Collectivite.conn' => $conn)));


            $this->loadModel('Compteur');
            $this->Compteur->useDbConfig = $conn;
            $caffaire = $this->Compteur->find('first', array('recursive' => -1, 'conditions' => array('Compteur.nom' => 'compteur_affaire')));
            $cdossier = $this->Compteur->find('first', array('recursive' => -1, 'conditions' => array('Compteur.nom' => 'compteur_dossier')));

            $this->request->data['Caffaire'] = $caffaire['Compteur'];
            $this->request->data['Cdossier'] = $cdossier['Compteur'];

            $this->loadModel('User');
            $this->User->setDataSource($conn);
            $this->loadModel('Desktop');
            $this->Desktop->setDataSource($conn);


            $scanDestDispRoles = $this->Desktop->find('list', array('conditions' => array('Desktop.group_id' => array(DISP_GID))));
            $this->set('scanDestDispRoles', $scanDestDispRoles);
            $scanDestInitRoles = $this->Desktop->find('list', array('conditions' => array('Desktop.group_id' => array(INIT_GID))));
            $this->set('scanDestInitRoles', $scanDestInitRoles);

            // Gestion par bureau
            $this->loadModel('Desktopmanager');
            $this->Desktopmanager->setDataSource($conn);
            $scanDesktops = $this->Desktopmanager->find('list', array('conditions' => array('Desktopmanager.isdispatch' => true)));
            $this->set('scanDesktops', $scanDesktops);

            $hasLogo = !empty($this->request->data['Collectivite']['logo']) ? true : false;
            $this->set('hasLogo', $hasLogo);
        }
		
        $conn = $this->Session->read('Auth.User.connName');
        $this->loadModel('Collectivite');
        $this->set('collectivite', $this->Collectivite->find('first', array('conditions' => array('Collectivite.conn' => $conn))));
    }

    /**
     *
     */
    /* public function synchronize() {
      $ldapconn = ldap_connect(LDAP_Configure::read('HOST'), PORT)
      or die("Impossible de se connecter au serveur LDAP {LDAP_Configure::read('HOST')}");
      if ($ldapconn) {
      // bind with appropriate dn to give update access
      $r = ldap_bind($ldapconn, MANAGER, LDAP_PASS);
      if (!$r)
      die("ldap_bind failed<br>");

      $dn = "ou=users,dc=adullact,dc=org";
      $filter = "(|(sn=*))";
      $justthese = array(MAIL, COMMON_NAME, UNIQUE_ID, PASSWORD_USER);
      $sr = ldap_search($ldapconn, $dn, $filter, $justthese);
      $users = ldap_get_entries($ldapconn, $sr);

      foreach ($users as $user) {
      if (isset($user[UNIQUE_ID][0]))
      $login = $user[UNIQUE_ID][0];

      if (isset($user[PASSWORD_USER][0]))
      $password = $user[PASSWORD_USER][0];
      else
      unset($password);

      if (isset($login) && isset($password)) {
      if (isset($user[MAIL][0]))
      $mail = $user[MAIL][0];
      else
      $mail = "";

      if (isset($user[COMMON_NAME][0])) {
      $cn = $user[COMMON_NAME][0];
      $prenom = substr($cn, 0, strpos($cn, ' '));
      $nom = substr($cn, strpos($cn, ' '), strlen($cn));
      }
      else
      $cn = "";

      $data = $this->User->findAll("User.login = '$login'");
      if (empty($data)) {
      $this->User->create();
      $this->data['User']['id'] = '';
      $this->data['User']['nom'] = $nom;
      $this->data['User']['prenom'] = $prenom;
      $this->data['User']['login'] = $login;
      $this->data['User']['mail'] = $mail;
      $pwd = substr($password, 5, strlen($password));
      $mdp = unpack("H*", $pwd);
      $this->data['User']['password'] = $mdp[1];
      $this->User->save($this->data);
      }
      }
      }
      ldap_close($ldapconn);
      } else {
      echo "Unable to connect to LDAP server";
      }
      }
     *
     */

    public function deleteLogo($collectiviteId = null) {
        if ($collectiviteId != null) {
            $this->Collectivite->id = $collectiviteId;
            $collectivite = $this->Collectivite->find(
                    'first', array(
                'fields' => array(
                    'Collectivite.id',
                    'Collectivite.logo'
                ),
                'conditions' => array(
                    'Collectivite.id' => $collectiviteId
                ),
                'contain' => false
                    )
            );
            $this->autoRender = false;

            $this->Collectivite->updateAll(
                    array('Collectivite.logo' => null), array('Collectivite.id' => $collectiviteId)
            );
//            debug($collectivite);
//			$this->set('logo_url', $collectivite['Collectivite']['logo']);
        }
    }

}

?>