<?php

/**
 * Gabaritsdocuments
 *
 * Gabaritsdocuments controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 7
 * @author Arnaud AUZOLAT
 * @copyright Initié par ADULLACT - Développé par ADULLACT Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		Controller
 */
class GabaritsdocumentsController extends AppController {

    /**
     * Controller name
     *
     * @access public
     * @var string
     */
    public $name = 'Gabaritsdocuments';
    public $uses = array('Gabaritdocument');

    /**
     * Gestion des opérations interface graphique)
     *
     * @logical-group Scanemails
     * @user-profil Admin
     *
     * @access public
     * @return void
     */
    public function index() {
        $this->set('ariane', array(
            '<a href="/environnement/index/0/admin">' . __d('menu', 'Administration', true) . '</a>',
            __d('menu', 'gabaritsdocuments', true)
        ));
    }

    /**
     * Controller components
     *
     * @var array
     * @access public
     */
    public $components = array('Linkeddocs');

    /**
     * Etat d'envoi des fichiers
     *
     * @access private
     * @var array
     */
    private $_uploadStatus = array(
        'NO_UPLOAD' => 0,
        'UPLOAD_START' => 1,
        'UPLOAD_END' => 2
    );

    /**
     * Types MIME et extensions autorisés
     *
     * @access private
     * @var array
     */
    private $_allowed = array(
        'ott' => array(
            'application/vnd.oasis.opendocument.text' => 'odt',
            'application/vnd.oasis.opendocument.text-template' => 'ott',
            'application/force-download' => 'odt'
        )
    );

    /**
     * Ajout d'un modèle de gabarit
     *
     * @logical-group Flux
     * @logical-group Context
     * @logical-group Modèles d'accusés de réception
     * @user-profile Admin
     *
     * @access public
     * @param integer $soustype_id
     * @return void
     */
    public function add() {
        if (empty($this->request->data)) {
            $options = array();
            $warnings = array();

            $this->set('warnings', $warnings);
            $this->set('options', $options);
        } else {

            if ($this->request->is('post')) {
                $upload = $this->_uploadStatus['NO_UPLOAD'];

                $gabarit = $this->request->data;
//debug($gabarit);
                //traitement de l upload des fichiers
                if ($_FILES['myfile']['error'] != UPLOAD_ERR_NO_FILE) {
                    $upload = $this->_uploadStatus['UPLOAD_START'];
                    $ext = '';
                    if (preg_match("/\.([^\.]+)$/", $_FILES['myfile']['name'], $matches)) {
                        $ext = $matches[1];
                    }
//debug($ext);
//debug($_FILES);
                    if (!in_array($_FILES['myfile']['type'], array_keys($this->_allowed[$this->request->data['Gabaritdocument']['format']]))) {
                        $message = __d('default', 'Upload.error.format') . (Configure::read('debug') > 0 ? ' (mime : ' . $_FILES['myfile']['type'] . ')' : '');
                    } else {
                        if ($ext != $this->_allowed[$this->request->data['Gabaritdocument']['format']][$_FILES['myfile']['type']]) {
                            $message = __d('default', 'Upload.error.bad_mime_ext') . (Configure::read('debug') > 0 ? ' (ext :  ' . $ext . ' - mime : ' . $_FILES['myfile']['type'] . ')' : '');
                        } else {
                            $upload = $this->_uploadStatus['UPLOAD_END'];
                            $gabarit['Gabaritdocument']['size'] = $_FILES['myfile']['size'];
                            $gabarit['Gabaritdocument']['ext'] = $ext;
                            $gabarit['Gabaritdocument']['mime'] = $_FILES['myfile']['type'];
                        }
                    }
                }

                //enregistrement des infos en base si pas  d upload ou si l upload est terminé
                $create = false;
                if ($upload != $this->_uploadStatus['UPLOAD_START']) {

                    $this->Gabaritdocument->create($gabarit);
                    $gabaritsaved = $this->Gabaritdocument->save();

                    if (!empty($gabaritsaved)) {
                        $create = true;
                        $message = __d('default', 'save.ok');
                    }

                    //suppression du fichier si un fichier est uploadé
                    if ($_FILES['myfile']['error'] != UPLOAD_ERR_NO_FILE) {
                        unlink($_FILES['myfile']['tmp_name']);
                    }
                }
                $this->set('create', $create);
                $this->set('message', $message);
            }
        }
    }

    /**
     * Edition d'un gabarit
     *
     * @logical-group Gabaritsdocuments
     * @user-profile Admin
     *
     * @access public
     * @param integer $id identifiant du type
     * @throws NotFoundException
     * @return void
     */
    public function edit($id) {

        if (!empty($this->request->data)) {
            $this->Jsonmsg->init();
            $this->Gabaritdocument->create();
            $gabarits = $this->request->data;


            if ($this->Gabaritdocument->save($gabarits)) {
                $this->Jsonmsg->valid();
            }
            $this->Jsonmsg->send();
        } else {
            $gabarits = $this->Gabaritdocument->find(
                    'first', array(
                'conditions' => array(
                    'Gabaritdocument.id' => $id
                ),
                'contain' => false
                    )
            );

            if (empty($gabarits)) {
                throw new NotFoundException();
            }
        }

        $this->set('gabarits', $gabarits);
    }

    /**
     * Suppression d'une opération
     *
     * @logical-group Gabaritsdocuments
     * @user-profil Admin
     *
     * @access public
     * @param integer $id identifiant du scanemail
     * @return void
     */
    public function delete($id = null) {
        $this->Jsonmsg->init(__d('default', 'delete.error'));
        $this->Gabaritdocument->begin();
        if ($this->Gabaritdocument->delete($id)) {
            $this->Gabaritdocument->commit();
            $this->Jsonmsg->valid(__d('default', 'delete.ok'));
        } else {
            $this->Gabaritdocument->rollback();
        }
        $this->Jsonmsg->send();
    }

    /**
     * Récupération de la liste des opérations (ajax)
     *
     * @logical-group Gabaritsdocuments
     * @user-profil Admin
     *
     * @access public
     * @return void
     */
    public function getGabaritsdocuments() {
        $gabaritsdocuments_tmp = $this->Gabaritdocument->find(
                "all", array(
            'contain' => false,
            'order' => array('Gabaritdocument.name')
                )
        );

        $gabaritsdocuments = array();
        foreach ($gabaritsdocuments_tmp as $i => $item) {
            $item['right_edit'] = true;
            $item['right_delete'] = true;
            $gabaritsdocuments[] = $item;
        }
        $this->set(compact('gabaritsdocuments'));
    }

    /**
     * Téléchargement d'un modèle d'accusée de réception
     *
     * @logical-group Flux
     * @logical-group Context
     * @logical-group Modèles d'accusés de réception
     * @user-profile User
     *
     * @access public
     * @param integer $id
     * @return void
     */
    public function download($id) {
        $gabarit = $this->Gabaritdocument->find('first', array('conditions' => array('Gabaritdocument.id' => $id), 'recursive' => -1));
        if (empty($gabarit)) {
            throw new NotFoundException();
        }

//debug(666);
//die();
        $ext = $gabarit['Gabaritdocument']['ext'];
        Configure::write('debug', 0);
        header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
        header("Content-type: {$gabarit['Gabaritdocument']['mime']}");
        header('Content-Length: ' . strlen($gabarit['Gabaritdocument']['content'])); // FIXME: length
        header("Content-Disposition: attachment; filename=\"{$gabarit['Gabaritdocument']['name']}.{$ext}\"");

//header('Pragma: public');
//    header('Expires: 0');
//    header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
//    header('Content-Description: File Transfer');
//    header('Content-Type: text/text');
//    header('Content-Disposition: attachment; filename="'.$gabarit['Gabaritdocument']['name'].'.'.$ext.'"');
//    header('Content-Transfer-Encoding: binary');
//    header('Cache-Control: max-age=0');
        echo $gabarit['Gabaritdocument']['content'];
        die();
//		$this->autoRender = false;
//		Configure::write('debug', 0);
//
//		header("Pragma: public");
//		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
//
//		header("Content-type: application/octet-stream");
//		header("Content-Disposition: attachment; filename=\"" . Inflector::slug($gabarit['Gabaritdocument']["name"]) . '.' . $gabarit['Gabaritdocument']['ext'] . "\"");
//		header("Content-length: " . $gabarit['Gabaritdocument']['size']);
//
//		print $gabarit['Gabaritdocument']['content'];
//		exit();
//        ignore_user_abort(true);
//        set_time_limit(0); // disable the time limit for this script
//
//        
//        $ext = $gabarit['Gabaritdocument']['ext'];
//        $path = APP . DS . WEBROOT_DIR . DS . 'files' . DS . "{$gabarit['Gabaritdocument']['name']}.{$ext}";
//        $file = file_put_contents( $path, $gabarit['Gabaritdocument']['content']);
//        $handle = fopen($path, "r");
//
//        $fullPath = $path;
//
//        if ($fd = fopen ($fullPath, "r")) {
//            $fsize = filesize($fullPath);
//            $path_parts = pathinfo($fullPath);
//debug($path_parts);
//die();
////
//
////            $ext = strtolower($path_parts["extension"]);
//var_dump($ext);
//die();
//            switch ($ext) {
//                case "pdf":
//                    header("Content-type: application/pdf");
//                    header("Content-Disposition: attachment; filename=\"".$path_parts["basename"].'.'.$ext."\""); // use 'attachment' to force a file download
//                break;
//                case "odt":
//                    header("Pragma: public");
//                    header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
//                    header("Content-type: application/vnd.oasis.opendocument.text");
////                    header("Content-type: application/octet-stream");
////                header('Content-Transfer-Encoding: binary');
//                    header("Content-Disposition: attachment; filename=\"".$path_parts["basename"].'.'.$ext."\"");
//                break;
//                // add more headers for other content types here
//                default;
//                    header("Pragma: public");
//                    header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
//                    header("Content-type: application/octet-stream");
//                    header('Content-Transfer-Encoding: binary');
//                    header("Content-Disposition: attachment; filename=\"".$path_parts["basename"].'.'.$ext."\"");
//                break;
//            }
//            header("Content-length: $fsize");
//            header("Cache-control: private"); //use this to open files directly
//            while(!feof($fd)) {
//                $buffer = fread($fd, 2048);
//                echo $buffer;
//            }
//        }
//        fclose ($fd);
//        unlink($fullPath);
//        exit;
    }

}

?>