<?php

/**
 * Fiches de contact
 *
 * OrganismesController controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 7
 * @author Stéphane Sampaio
 * @copyright Initié par ADULLACT - Développé par ADULLACT Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		Controller
 */
class OrganismesController extends AppController {

    /**
     * Controller uses
     *
     * @var array
     */
    public $uses = array('Organisme');

    /**
     *
     */
    public function beforeFilter() {
        parent::beforeFilter();
        $this->Auth->allow('import_organismes', 'import', 'export');
    }

    protected function _setOptions() {
        $options = array();
        $options = $this->Organisme->Contact->Addressbook->listTypesVoie();

//        $civilite = array('Monsieur' => 'Monsieur', 'Madame' => 'Madame', 'Monsieur et Madame' => 'Monsieur et Madame');
        $civilite = array(1 => 'Monsieur', 2 => 'Madame', 3 => 'Monsieur et Madame');

        $this->set('bans', $this->Organisme->Ban->find(
                        'all', array(
                    'order' => array('Ban.id ASC'),
                    'contain' => array(
                        'Bancommune' => array(
                            'order' => 'Bancommune.name ASC'
                        )
                    )
                        )
                )
        );
        $this->set('addressbooks', $this->Organisme->Contact->Addressbook->find('list', array('conditions' => array('Addressbook.active' => true))));
//        debug($options);
        $this->set(compact('options', 'civilite'));

        if (Configure::read('Conf.SAERP')) {
            $this->set('activites', $this->Organisme->Activite->find('list', array('order' => array('Activite.name ASC'), 'conditions' => array('Activite.active' => true))));
            $this->set('operations', $this->Organisme->Operation->find('list', array('order' => array('Operation.name ASC'), 'conditions' => array('Operation.active' => true))));
            $this->set('events', $this->Organisme->Contact->Event->find('list', array('order' => array('Event.name ASC'), 'conditions' => array('Event.active' => true))));
            $this->set('fonctions', $this->Organisme->Contact->Fonction->find('list', array('order' => array('Fonction.name ASC'), 'conditions' => array('Fonction.active' => true))));
        }
    }

    /**
     * Ajout d'une fiche de contact
     *
     * @logical-group Contact
     * @logical-group Fiche de contact
     * @user-profile Admin
     *
     * @logical-used-by Flux > Ajout d'un flux
     * @user-profile User
     *
     * @logical-used-by Flux > Edition d'un flux
     * @user-profile User
     *
     * @access public
     * @param integer $contactId
     * @return void
     */
    public function add() {
        if (!empty($this->request->data)) {
            $this->Jsonmsg->init();
//            $options = $this->Organisme->Bancommune->find('list');
//            $ville = Hash::get($options, Hash::get($this->request->data, 'Organisme.bancommune_id'));
//            $this->request->data['Organisme']['adressecomplete'] = Hash::get( $this->request->data, 'Organisme.numvoie' ).' '.$typevoie.' '.Hash::get( $this->request->data, 'Organisme.nomvoie' );
//            $this->request->data['Organisme']['adressecomplete'] = Hash::get($this->request->data, 'Organisme.numvoie') . ' ' . Hash::get($this->request->data, 'Organisme.banadresse') . ' ' . Hash::get($this->request->data, 'Organisme.cp') . ' ' . $ville;

            if (!empty($this->request->data['Organisme']['banadresse'])) {
                $adressecomplete = Hash::get($this->request->data, 'Organisme.numvoie') . ' ' . Hash::get($this->request->data, 'Organisme.banadresse');
            } else {
                $adressecomplete = Hash::get($this->request->data, 'Organisme.numvoie') . ' ' . Hash::get($this->request->data, 'Organisme.nomvoie');
            }
            $this->request->data['Organisme']['adressecomplete'] = $adressecomplete;
            $this->Organisme->create($this->request->data);
            $success = $this->Organisme->save();

            if ($success) {
                if ($this->request->data['Organisme']['infoscontact'] == 1) {
                    $this->request->data['Contact']['addressbook_id'] = $this->request->data['Organisme']['addressbook_id'];
                    $this->request->data['Contact']['organisme_id'] = $this->Organisme->id;
                    $this->request->data['Contact']['adresse'] = $adressecomplete;
                } else {
                    $this->request->data['Contact']['addressbook_id'] = $this->request->data['Organisme']['addressbook_id'];
                    $this->request->data['Contact']['organisme_id'] = $this->Organisme->id;
                    $this->request->data['Contact']['adresse'] = $adressecomplete;
                    $this->request->data['Contact']['name'] = ' Sans contact';
                    $this->request->data['Contact']['nom'] = ' 0Sans contact';
                }

                $this->Organisme->Contact->create($this->request->data['Contact']);
                $success = $this->Organisme->Contact->save() && $success;
            }
            if ($success) {
                $this->Jsonmsg->valid();
            } else {
                return false;
            }
            $this->Jsonmsg->send();
        }
        $this->_setOptions();
    }

    /**
     *
     */

    /**
     * Edition d'une fiche de contact
     *
     * @logical-group Contact
     * @logical-group Fiche de contact
     * @user-profile Admin
     *
     * @access public
     * @param integer $id
     * @return void
     */
    public function edit($id = null) {
        $this->Organisme->recursive = -1;


        if (!empty($this->request->data)) {
//            $options = $this->Organisme->Bancommune->find('list');
//            $ville = Hash::get($options, Hash::get($this->request->data, 'Organisme.bancommune_id'));
//            $this->request->data['Organisme']['adressecomplete'] = Hash::get( $this->request->data, 'Organisme.numvoie' ).' '.$typevoie.' '.Hash::get( $this->request->data, 'Organisme.nomvoie' );
//            $this->request->data['Organisme']['adressecomplete'] = Hash::get($this->request->data, 'Organisme.numvoie') . ' ' . Hash::get($this->request->data, 'Organisme.banadresse') . ' ' . Hash::get($this->request->data, 'Organisme.cp') . ' ' . $ville;
            if (!empty($this->request->data['Organisme']['banadresse'])) {
                $adressecomplete = Hash::get($this->request->data, 'Organisme.numvoie') . ' ' . Hash::get($this->request->data, 'Organisme.banadresse');
            } else {
                $adressecomplete = Hash::get($this->request->data, 'Organisme.numvoie') . ' ' . Hash::get($this->request->data, 'Organisme.nomvoie');
            }
            $this->request->data['Organisme']['adressecomplete'] = $adressecomplete;
            $this->Jsonmsg->init();
//            $this->Organisme->create($this->request->data);
            if ($this->Organisme->save($this->request->data)) {
                $this->Jsonmsg->valid();
            }
            $this->Jsonmsg->send();
        } else if ($id != null) {
            $querydata = array(
                'conditions' => array(
                    'Organisme.id' => $id
                ),
                'contain' => false
            );
            if (Configure::read('Conf.SAERP')) {
                $querydata['contain'] = array(
                    'Operation',
                    'Activite'
                );
            }
            $this->request->data = $this->Organisme->find('first', $querydata);
        }
        $this->_setOptions();
        $this->set('id', $id);
    }

    /**
     * Suppression d'une fiche de contact (soft delete - delete)
     *
     * @logical-group Contact
     * @logical-group Fiche de contact
     * @user-profile Admin
     *
     * @access public
     * @param integer $id
     * @return void
     */
    public function delete($id = null, $real = true) {
        if ($id != null) {
            $contactInOrganisme = $this->Organisme->Contact->find('count', array(
                'conditions' => array(
                    'organisme_id' => $id
                )
            ));
            if ($real) {
                $this->Jsonmsg->init(__d('default', 'delete.error'));
                if ($contactInOrganisme == 0) {
                    if ($this->Organisme->delete($id, true)) {
                        $this->Jsonmsg->valid($this->Organisme->contain());
                        $this->Jsonmsg->valid(__d('default', 'delete.ok'));
                    }
                } else {
                    $this->Jsonmsg->error('Cet organisme n\'est pas vide, pas possible de le supprimer!');
                }
            } else {
                $this->Jsonmsg->init();
                $this->Organisme->id = $id;
                if ($contactInOrganisme == 0) {
                    if ($this->Organisme->saveField('active', false)) {
                        $this->Jsonmsg->valid();
                    }
                } else {
                    $this->Jsonmsg->error('Cet organisme n\'est pas vide, pas possible de le supprimer!');
                }
            }
            $this->Jsonmsg->send();
        }
    }

    /**
     * Fonction en ajax permettant de remonter les adresses suite à la sélection d'une commune
     * @param type $bancommune_id
     * @return type
     */
    public function getAdresses($bancommune_id) {
        $this->autoRender = false;
        $adresses = $this->Organisme->Bancommune->Banadresse->find(
                'all', array(
            'fields' => array(
                'DISTINCT Banadresse.nom_afnor'
            ),
            'conditions' => array(
                'Banadresse.bancommune_id' => $bancommune_id,
                array(
                    'Banadresse.nom_afnor IS NOT NULL',
                    'Banadresse.nom_afnor <>' => '',
                )
            ),
            'recursive' => -1,
            'order' => array('Banadresse.nom_afnor')
                )
        );

        $content = json_encode($adresses);
        header('Pragma: no-cache');
        header('Cache-Control: no-store, no-cache, max-age=0, must-revalidate');
        header('Content-Type: text/x-json');
        header("X-JSON: {$content}");
        Configure::write('debug', 0);
        echo $content;
        return;
    }

    /**
     * Récupération de la liste des organismes
     *
     * @logical-group Organisme
     * @user-profile Admin
     *
     * @access public
     * @param integer $addressbook_id identifiant du carnet d'adresse
     * @return void
     */
    public function get_organismes($orgId = null, $clear = null) {
        // On doit pouvoir obtenir les résultats dès le premier accès à la page
        if (!isset($this->request->data['Organisme'])) {
            $this->request->data = Set::merge(
                            $this->Session->read('paginationModel.rechercheOrganisme'), array('Organisme' => array('active' => true, 'id' => $orgId))
            );
            if ($clear != null) {
                $this->request->data = array(
                    'Organisme' => array(
                        'SearchContact' => array(
                            'name' => '',
                            'nom' => '',
                            'prenom' => '',
                            'active' => '1'
                        )
                    )
                );
            }
        }
        if ($orgId != null) {
            $organismes = array();
            if (empty($this->request->data)) {
                $this->Contact->recursive = -1;

                $querydata = array(
                    'conditions' => array(
                        'Contact.organisme_id' => $orgId
                    ),
                    'contain' => false,
                    'limit' => 20
                );
            } else {
                $this->Session->write('paginationModel.rechercheOrganisme', $this->request->data);
                $querydata = $this->Organisme->Contact->search($this->request->data['Organisme'], null, $orgId);
            }
            $this->paginate = $querydata;
            $contacts = $this->paginate($this->Organisme->Contact);

            foreach ($contacts as $i => $contact) {
                $contact['right_edit'] = true;
                $contact['right_delete'] = true;
                if (Configure::read('Conf.SAERP')) {
                    $contact['right_export'] = true;
                }
                $contacts[$i] = $contact;

                // Ajout du nom de l'organisme dans le tableau reprenant les contacts
                if (!empty($contact['Organisme']['name'])) {
                    $contacts[$i]['Contact']['organisme_name'] = $contact['Organisme']['name'];
                } else {
                    $contacts[$i]['Contact']['organisme_name'] = '';
                }
            }

            $this->set('contacts', $contacts);
//            $this->set('addressbook_id', $addressbook_id);
            $this->set('orgId', $orgId);

            $organisme = $this->Organisme->find(
                    'first', array(
                'conditions' => array(
                    'Organisme.id' => $orgId
                ),
                'recursive' => -1,
                'contain' => false
                    )
            );
            $organisme_name = $organisme['Organisme']['name'];
            $addressbook_id = $organisme['Organisme']['addressbook_id'];
            $this->set('organisme_name', $organisme_name);
            $this->set('addressbook_id', $addressbook_id);
            $this->set('organisme_id', $orgId);
            $this->_setOptions();
        }
    }

    /**
     *
     */
    public function getHomonymsOrgs() {
        Configure::write('debug', 0);
        $this->autoRender = false;
        $return = array();
        if ($this->RequestHandler->isAjax() && !empty($this->request->data['Organisme']['name'])) {
            $return = $this->Organisme->getHomonyms($this->request->data['Organisme']['name']);
        }
        echo json_encode($return);
    }

    /**
     * Analyse le fichier importé, et importation de contact et de fiches de contact à partir d'un fichier vcard/csv
     *
     * @logical-group Contact
     * @user-profile Admin
     *
     * @access public
     * @return void
     */
    public function import($addressbook_id) {
        $this->autoRender = false;
        $return = array();
//var_dump($this->request->data['Import']);
        if (!empty($this->request->data)) {
            $ex = $this->request->data['Import']['format'];
//var_dump($this->request->data);
            //import fichier
            if (!empty($this->request->data['Import']['fileorg']) && empty($this->request->data['Import']['addressbook_id'])) {
//var_dump($this->request->data['Import']);
                $tmpName = $this->request->data['Import']['fileorg']['tmp_name'];
                $filename = TMP . $this->request->data['Import']['fileorg']['name'];
                $extension = pathinfo($filename, PATHINFO_EXTENSION);
                if (move_uploaded_file($tmpName, $filename) && $extension == $ex) {
                    $return['situation'] = true;
                    switch ($extension) {
                        case 'csv':
                            $return['filename'] = $filename;
                            $return['message'] = $this->Organisme->Addressbook->importOrganismeCsv($filename);
                            break;
                    }
                } else {
                    $return['situation'] = false;
                    $return['message'] = "L'extension de votre fichier est incorrect. Veuillez sélectionner un autre fichier.";
                }
                //enregistrer dans un carnet exist      
            } else if (!empty($this->request->data['Import']['fileorg']) && !empty($this->request->data['Import']['addressbook_id'])) {
//var_dump($this->request->data['Import']);
                $tmpName = $this->request->data['Import']['fileorg']['tmp_name'];
                $filename = TMP . $this->request->data['Import']['fileorg']['name'];
                $extension = pathinfo($filename, PATHINFO_EXTENSION);
                if (move_uploaded_file($tmpName, $filename) && $extension == $ex) {
                    $return['situation'] = true;
                    switch ($extension) {
                        case 'csv':
                            $return['filename'] = $filename;
                            $return['message'] = $this->Organisme->Addressbook->importOrganismeCsv($filename);
                            break;
                    }
                } else {
                    $return['situation'] = false;
                    $return['message'] = "L'extension de votre fichier est incorrect. Veuillez sélectionner un autre fichier.";
                }
                //enregistrer dans un carnet exist      
            } else if (!empty($this->request->data['Import']['update_file']) && (!empty($this->request->data['Import']['addressbook_id']) || !empty($addressbook_id) )) {
//var_dump(2);
//var_dump($this->request->data['Import']);die();
                $filename = TMP . $this->request->data['Import']['update_file'];
                $return['filename'] = $filename;
                $return['message'] = $this->Organisme->Addressbook->importContactCsv($filename);
                switch ($ex) {
                    case 'csv':
                        if ($this->Organisme->Addressbook->importOrganismeCsv($this->request->data['Import']['update_file'], $this->request->data['Import']['addressbook_id'], null)) {
                            $return['alert'] = __d('default', 'save.ok');
                        } else {
                            $return['alert'] = __d('default', 'save.error');
                        }
                        break;
                }
                if (is_file($this->request->data['Import']['update_file'])) {
                    unlink($this->request->data['Import']['update_file']);
                }
                //nouveau carnet
            } else if (!empty($this->request->data['Import']['update_file']) && !empty($this->request->data['Import']['carnet_nom'])) {
//var_dump(3);
                $addressbook = array();
                $addressbook = array(
                    'Addressbook' => array(
                        'name' => $this->request->data['Import']['carnet_nom'],
                        'description' => $this->request->data['Import']['carnet_description']
                    )
                );
                $nb = $this->Organisme->Addressbook->find('count', array(
                    'conditions' => array(
                        'Addressbook.name' => $this->request->data['Import']['carnet_nom']
                    )
                ));
                if (!$nb > 0) {
                    $this->Organisme->Addressbook->begin();
                    $this->Organisme->Addressbook->create();
                    $savedAddressbook = $this->Organisme->Addressbook->save($addressbook);
                    $saved = !empty($savedAddressbook);
                    if ($saved) {
                        $this->Organisme->Addressbook->commit();
                        switch ($ex) {
                            case 'csv':
                                if ($this->Organisme->Addressbook->importOrganismeCsv($this->request->data['Import']['update_file'], $savedAddressbook['Addressbook']['id'], null)) {
                                    $return['alert'] = __d('default', 'save.ok');
                                } else {
                                    $return['alert'] = __d('default', 'save.error');
                                }
                                break;
                        }
                        if (is_file($this->request->data['Import']['update_file'])) {
                            unlink($this->request->data['Import']['update_file']);
                        }
                    } else {
                        $this->Organisme->Addressbook->rollback();
                        $return['alert'] = __d('default', 'save.error');
                    }
                }
            }
        }
        return json_encode($return);
    }

    /**
     * chargement la page import_contacts
     *
     * @logical-group Contact
     * @user-profile Admin
     *
     * @access public
     * @return void
     */
    public function import_organismes($update = false) {
        $this->set('addressbooks', $this->Organisme->Addressbook->find('list', array('conditions' => array('Addressbook.active' => 1), 'order' => array('Addressbook.name'))));
    }

    /**
     * Export des carnets d'adresse au format CSV
     *
     * @logical-group Addressbooks
     * @user-profile Admin
     *
     * @access public
     * @return void
     */
    public function export($organisme_id = null) {
        ini_set('memory_limit', '3000M');
//        $this->autoRender = false;
        $name = Hash::get($this->request->params['named'], 'SearchOrganisme__name');
        $addressbook = Hash::get($this->request->params['named'], 'SearchOrganisme__addressbook_id');
        $organismeActif = Hash::get($this->request->params['named'], 'SearchOrganisme__active');
//debug($this->request->params);
//die();
        $conditions = array();
        $conditions = array(
            'Organisme.active' => true
        );
        if (!empty($this->request->params)) {

            if (isset($name) && !empty($name)) {
                $conditions[] = 'Organisme.name ILIKE \'' . $this->Organisme->wildcard('%' . $name . '%') . '\'';
            }

            if (isset($organismeActif) && !empty($organismeActif)) {
                $conditions[] = array('Organisme.active' => $organismeActif);
            }
        }

        $querydata = array(
            'conditions' => array(
                $conditions
            ),
            'order' => array(
                'Organisme.name ASC'
            ),
            'contain' => array(
                'Activite',
                'Operation',
                'Contact' => array(
                    'Fonction',
                    'Event',
                    'Operation'
                )
            )
        );

        $events = $this->Organisme->Contact->Event->find('list', array('conditions' => array('Event.active' => true)));
        $eventColumns = array();
        foreach ($events as $eventsIds => $eventName) {
            $eventColumns[$eventsIds] = strtoupper($eventName);
        }
        $this->set('eventColumns', $eventColumns);

        if (!empty($organisme_id)) {
            $querydata['conditions'] = array_merge(
                    $querydata['conditions'], array(
                'Organisme.id' => $organisme_id
                    )
            );

            $results = $this->Organisme->find('first', $querydata);
            $civiliteOptions = array(
                '1' => 'Monsieur',
                '2' => 'Madame',
                '3' => 'Monsieur et Madame',
                '' => ''
            );


//            foreach( $results['Organisme'] as $o => $organisme ) {
            $results['Organisme']['activites'] = implode(' - ', Hash::extract($results, 'Activite.{n}.name'));
            $results['Organisme']['operationsorg'] = implode(' - ', Hash::extract($results, 'Operation.{n}.name'));
            $i = 0;
            foreach ($results['Contact'] as $c => $contact) {
//debug($contact);
//die();
                if (!$i > 0) {
                    $civilite = $civiliteOptions[$contact['civilite']];
                    $results['Contact'][$c]['civilite'] = $civilite;
                }
            }
            foreach ($eventColumns as $eventID => $eventName) {
                foreach ($results['Contact'] as $c => $contact) {
//debug($contact);
//die();
                    if (!$i > 0) {
                        $civilite = $civiliteOptions[$contact['civilite']];
                        $results['Contact'][$c]['civilite'] = $civilite;
                    }
                    if (!empty($contact['Operation'])) {
                        $results['Contact'][$c]['operationscontact'] = implode(' - ', Hash::extract($contact, 'Operation.{n}.name'));
                    }
                    $results['Contact'][$c]['events'][$eventID] = '';
                    foreach ($contact['Event'] as $ev => $event) {
                        if ($eventID == $event['id']) {
                            $results['Contact'][$c]['events'][$eventID] = $eventName;
                        }
                    }
                    if (!empty($contact['observations'])) {
                        $results['Contact'][$c]['observations'] = preg_replace("/[\n\r\"]/", " ", $contact['observations']);
                    }
                }
                $i++;
            }
//debug($results);
//die();
//            }
//            $this->set( 'neweventColumns', $neweventColumns );
//debug($neweventColumns);
            $this->set('results', $results);
        } else {
            $querydata = $this->Organisme->search($this->request->data, null, array());
            $resultats_tmp = $this->Organisme->find('all', $querydata);
//debug($resultats_tmp);
//die();
            $resultats = array();
            foreach ($resultats_tmp as $i => $results) {
                $civiliteOptions = array(
                    '1' => 'Monsieur',
                    '2' => 'Madame',
                    '3' => 'Monsieur et Madame',
                    '' => ''
                );
                $results['Organisme']['activites'] = implode(' - ', Hash::extract($results, 'Activite.{n}.name'));
                $results['Organisme']['operationsorg'] = implode(' - ', Hash::extract($results, 'Operation.{n}.name'));
                $i = 0;
                foreach ($results['Contact'] as $c => $contact) {
                    if (!$i > 0) {
                        $civilite = $civiliteOptions[$contact['civilite']];
                        $results['Contact'][$c]['civilite'] = $civilite;
                    }
                }
                foreach ($eventColumns as $eventID => $eventName) {
                    foreach ($results['Contact'] as $c => $contact) {

                        if (!empty($contact['Operation'])) {
                            $results['Contact'][$c]['operationscontact'] = implode(' - ', Hash::extract($contact, 'Operation.{n}.name'));
                        }
                        $results['Contact'][$c]['events'][$eventID] = '';
                        if (isset($contact['Event'])) {
                            foreach ($contact['Event'] as $ev => $event) {
                                if ($eventID == $event['id']) {
                                    $results['Contact'][$c]['events'][$eventID] = $eventName;
                                }
                            }
                        }
                        if (!empty($contact['observations'])) {
                            $results['Contact'][$c]['observations'] = preg_replace("/[\n\r\"]/", " ", $contact['observations']);
                        }
                    }
                    $i++;
                }
                $resultats[] = $results;
            }
//debug($resultats);
//die();
            $this->set('results', $resultats);
        }
//        else {
//            $results = $this->Organisme->find( 'first', $querydata);
//        }
//debug($results);
//die();

        $this->_setOptions();
        $this->layout = '';
    }

    public function get_contact($organisme_id) {
        $querydata = array(
            'fields' => array(
                'Contact.id',
                'Contact.name'
            ),
            'conditions' => array(
                'Contact.organisme_id' => $organisme_id
            ),
            'order' => array(
                'Contact.name' => 'ASC'
            ),
            'contain' => FALSE,
            'limit' => 12
        );
        $this->paginate = $querydata;
        $contacts_tmp = $this->paginate($this->Organisme->Contact);
//        debug($contacts_tmp);
        foreach ($contacts_tmp as $i => $contact) {
            $contacts[$i]['name'] = $contact['Contact']['name'];
            $contacts[$i]['view'] = '<a href="#" class = "viewContact" id="contact_' . $contact["Contact"]["id"] . '" title="Voir"><i class="fa fa-search" aria-hidden="true" style="color:#428bca"></i></a>';
            $contacts[$i]['edit'] = '<a href="#" class="editContact" id="contact_' . $contact['Contact']["id"] . '" title="Modifier"><i class="fa fa-pencil-square-o" aria-hidden="true" style="color:#428bca"></i></a>';
            $contacts[$i]['delete'] = '<a href="#" class="deleteContact" id="contact_' . $contact['Contact']["id"] . '" title="Supprimer" organismeId ="' . $organisme_id . '"><i class="fa fa-trash" aria-hidden="true" style="color:red"></i></a>';
            $contacts[$i]['exportcsv'] = '<a href="#" class="exportContact" id="contact_' . $contact['Contact']["id"] . '" title="Exporter"><i class="fa fa-download" aria-hidden="true" style="color:#428bca"></i></a>';
        }
        if (!empty($contacts)) {
            $this->set('contacts', $contacts);
            $this->set('organisme_id', $organisme_id);
        } else {
            $this->set('contacts', array());
        }
        $this->_setOptions();
    }

    public function searchOrganismes() {
        if ($this->Session->read('searchAddressbook') != null) {
            $this->paginate = $this->Session->read('searchAddressbook');
        } else {
            $querydata = $this->request->data;
            $this->paginate = $querydata['organismes'];
            $this->Session->write('searchAddressbook', $this->paginate);
        }

        $organismes_tmp = $this->paginate($this->Organisme);
        $organismes = array();
        foreach ($organismes_tmp as $i => $organisme) {
            $organismes[$i]['name'] = $organisme['Organisme']['name'] . '<span style="color:#333333"> (' . $this->Organisme->Contact->getNombreContactByOrganismeId($organisme["Organisme"]["id"]) . ') <span>';
            $organismes[$i]['view'] = '<a href="#" class = "viewOrganisme" id="organisme_' . $organisme["Organisme"]["id"] . '" title="Voir"><i class="fa fa-search" aria-hidden="true" style="color:#428bca"></i></a>';
            $organismes[$i]['edit'] = '<a href="#" class="editOrganisme" id="organisme_' . $organisme["Organisme"]["id"] . '" title="Modifier"><i class="fa fa-pencil-square-o" aria-hidden="true" style="color:#428bca"></i></a>';
            $organismes[$i]['delete'] = '<a href="#" class="deleteOrganisme" id="organisme_' . $organisme["Organisme"]["id"] . '" title="Supprimer" addressbookId="' . $organisme["Organisme"]["addressbook_id"] . '"><i class="fa fa-trash" aria-hidden="true" style="color:red"></i></a>';
            $organismes[$i]['exportcsv'] = '<a href="#" class="exportOrganisme" id="organisme_' . $organisme["Organisme"]["id"] . '" title="Exporter"><i class="fa fa-download" aria-hidden="true" style="color:#428bca"></i></a>';
        }
        if (!empty($organismes)) {
            $this->set('organismes', $organismes);
        } else {
            $this->set('organismes', array());
        }
    }

}
