<?php

/**
 * Recherches
 *
 * Recherches controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 7
 * @author Stéphane Sampaio
 * @copyright Initié par ADULLACT - Développé par ADULLACT Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		Controller
 */
class RecherchesController extends AppController {

    /**
     * Controller name
     *
     * @var string
     * @access public
     */
    public $name = 'Recherches';

    /**
     * Controller uses
     *
     * @var array
     * @access public
     */
    public $uses = array('Recherche', 'Courrier', 'Ordreservice', 'Pliconsultatif');

    /**
     * Controller components
     *
     * @var array
     * @access public
     */
    public $components = array('DataAuthorized');

    /**
     * Controller helpers
     *
     * @var array
     * @access public
     */
    public $helpers = array('Csv', 'Paginator');

    //TODO: enlever le beforeFilter et régénérer les droits
    public function beforeFilter() {
        $this->Auth->allow(array('index', 'exportcsv', 'export')); //FIXME
        parent::beforeFilter();
    }

    protected function _setOptions() {
        $this->loadModel('Desktop');
        $desktopId = $this->Session->read('Auth.User.Courrier.Desktop.id');
        $daros = $this->Desktop->getDaros($desktopId);
        $this->set('taches', $this->Recherche->Tache->find('list', array('order' => array('Tache.name ASC'))));
        $this->set('circuits', $this->Recherche->Circuit->find('list', array('order' => array('Circuit.nom ASC'))));
        $this->set('dossiers', $this->Recherche->Dossier->find('list', array('order' => array('Dossier.name ASC'))));
        $this->set('affaires', $this->Recherche->Affaire->find('list', array('order' => array('Affaire.name ASC'))));
        $this->set('origineflux', $this->Courrier->Origineflux->find('list', array('order' => array('Origineflux.name ASC'))));
        $metas = $this->Recherche->Metadonnee->find('all', array('contain' => array('Selectvaluemetadonnee', 'Typemetadonnee')));
        $this->set('metas', $metas);
        $valueSelectMeta = array('no' => 'Non', 'yes' => 'Oui');
        $this->set('valueSelectMeta', $valueSelectMeta);
        $contacts = $this->Recherche->Contact->find('list', array('order' => 'Contact.name ASC'));
        $this->set('contacts', $contacts);
        $organismes = $this->Recherche->Organisme->find('list', array('order' => 'Organisme.name ASC'));
        $this->set('organismes', $organismes);
//			$this->set('types', $this->DataAuthorized->getAuthTypes(array('find' => 'list')));
        if (Configure::read('Use.AuthRights') && !Configure::read('Conf.SAERP')) {
            $this->set('types', $this->DataAuthorized->getAuthTypes(array('withSoustypes' => true, 'onlyActif' => false)));
        } else {
            $this->set('types', $this->Courrier->Soustype->Type->find('all', array('contain' => array('Soustype' => array('order' => 'Soustype.name ASC')), 'order' => array('Type.name ASC'), 'conditions' => array('Type.active' => 1), 'recursive' => -1)));
//            $this->set('types', $this->Courrier->Soustype->Type->find('list', array('order' => array('Type.name ASC'), 'recursive' => -1)));
        }
//			$this->set('soustypes', $this->DataAuthorized->getAuthSoustypes(array('find' => 'all')));
        $this->set('metadonnees', $this->DataAuthorized->getAuthMetadonnees(array('find' => 'list')));
        $affairesuiviepars = $this->Courrier->Desktop->getDesktopsByService('all');
        ksort($affairesuiviepars);
        $this->set('affairesuiviepars', $affairesuiviepars);
        // Adresse
        $this->set('bans', ClassRegistry::init('Ban')->find(
                        'all', array(
                    'order' => array('Ban.name ASC'),
                    'contain' => array(
                        'Bancommune' => array(
                            'order' => 'Bancommune.name ASC'
                        )
                    )
                        )
                )
        );
        $this->set(
                'etat', array(
            '-1' => 'Refusé',
            '0' => 'Validé',
            '1' => 'En cours de traitement',
            '2' => 'Clôturé'
                )
        );
        $services = $this->Recherche->Service->find('list', array('order' => 'Service.name ASC'));
        /////////////////////////////
        $this->set('services', $services);
        if (Configure::read('Conf.SAERP')) {
            $marches = $this->Courrier->Marche->find(
                    'list', array(
                'joins' => array(
                    $this->Courrier->Marche->join('Operation')
                ),
                'conditions' => array(
                    'Marche.active' => true,
                    'Operation.active' => true
                ),
                'fields' => array(
                    'Marche.numero'
                ),
                'order' => array('Marche.name ASC')
                    )
            );
            $this->set(compact('marches'));
            $os = $this->Courrier->Ordreservice->find(
                    'list', array(
                'fields' => array(
                    'Ordreservice.name'
                ),
                'order' => array('Ordreservice.name ASC')
                    )
            );
//debug($os);
            $plis = $this->Courrier->Pliconsultatif->find(
                    'list', array(
                'fields' => array(
                    'Pliconsultatif.name'
                ),
                'order' => array('Pliconsultatif.name ASC')
                    )
            );
            $typedocument = array(
                'OS' => 'Ordre de service',
                'LN' => 'Lettre de notification',
                'LC' => 'Lettre de commandes',
                'PLI' => 'Pli'
            );
            $this->set(compact('typedocument'));
            $this->set(compact('os', 'plis'));

            $this->Organisme = ClassRegistry::init('Organisme');
            $this->set('activites', $this->Organisme->Activite->find('list', array('order' => array('Activite.name ASC'), 'conditions' => array('Activite.active' => true))));
            $this->set('events', $this->Organisme->Contact->Event->find('list', array('order' => array('Event.name ASC'), 'conditions' => array('Event.active' => true))));
            $this->set('fonctions', $this->Organisme->Contact->Fonction->find('list', array('order' => array('Fonction.name ASC'), 'conditions' => array('Fonction.active' => true))));
        }

        $conn = $this->Session->read('Auth.User.connName');
        $this->loadModel('Collectivite');
        $this->set('collectivite', $this->Collectivite->find('first', array('conditions' => array('Collectivite.conn' => $conn))));
    }

    /**
     * Index method
     *
     * @access public
     * @return void
     */
    public function index() {
        $this->set('ariane', array(__d('menu', 'Recherche', true)));
    }

    /**
     * Formulaire de recherche (enregistrement de recherche)
     *
     * @logical-group Recherches
     * @user-profile User
     *
     * @access public
     * @param integer $id
     * @param string $forcemode
     * @return void
     */
    public function formulaire($id = null, $forcemode = null) {
        //enregistrement des critères de la recherche

        if (!empty($this->request->data)) {
            $this->Jsonmsg->init();
            $data = array();
            $data['Recherche'] = $this->request->data['Recherche'];
            if (!empty($this->request->data['Contact']['Contact']) || empty($this->request->data['Recherche']['id'])) {
                $data['Contact'] = array();
                foreach ($this->request->data['Contact']['Contact'] as $kcontact => $contact) {
                    $data['Contact']['Contact'][$kcontact] = $contact;
                }
            } else {
                $query = "DELETE FROM recherches_contacts WHERE recherche_id = " . $this->request->data('Recherche.id') . ";";
                $this->Recherche->query($query);
            }
            if (!empty($this->request->data['Type']['Type']) || empty($this->request->data['Recherche']['id'])) {
                foreach ($this->request->data['Type']['Type'] as $ktype => $type) {
                    $data['Type']['Type'][$ktype] = $type;
                }
            } else {
                $query = "DELETE FROM recherches_types WHERE recherche_id = " . $this->request->data['Recherche']['id'] . ";";
                $this->Recherche->query($query);
            }
            if (!empty($this->request->data['Soustype']['Soustype']) || empty($this->request->data['Recherche']['id'])) {
                foreach ($this->request->data['Soustype']['Soustype'] as $ksoustype => $soustype) {
                    $data['Soustype']['Soustype'][$ksoustype] = $soustype;
                }
            } else {
                $query = "DELETE FROM recherches_soustypes WHERE recherche_id = " . $this->request->data['Recherche']['id'] . ";";
                $this->Recherche->query($query);
            }
            if (!empty($this->request->data['Circuit']['Circuit']) || empty($this->request->data['Recherche']['id'])) {
                $data['Circuit'] = array();
                foreach ($this->request->data['Circuit']['Circuit'] as $kcircuit => $circuit) {
                    $data['Circuit']['Circuit'][$kcircuit] = $circuit;
                }
            } else {
                $query = "DELETE FROM recherches_circuits WHERE recherche_id = " . $this->request->data['Recherche']['id'] . ";";
                $this->Recherche->query($query);
            }
            if (!empty($this->request->data['Dossier']['Dossier']) || empty($this->request->data['Recherche']['id'])) {
                $data['Dossier'] = array();
                foreach ($this->request->data['Dossier']['Dossier'] as $kdossier => $dossier) {
                    $data['Dossier']['Dossier'][$kdossier] = $dossier;
                }
            } else {
                $query = "DELETE FROM recherches_dossiers WHERE recherche_id = " . $this->request->data['Recherche']['id'] . ";";
                $this->Recherche->query($query);
            }
            if (!empty($this->request->data['Affaire']['Affaire']) || empty($this->request->data['Recherche']['id'])) {
                $data['Affaire'] = array();
                foreach ($this->request->data['Affaire']['Affaire'] as $kaffaire => $affaire) {
                    $data['Affaire']['Affaire'][$kaffaire] = $affaire;
                }
            } else {
                $query = "DELETE FROM recherches_affaires WHERE recherche_id = " . $this->request->data['Recherche']['id'] . ";";
                $this->Recherche->query($query);
            }
            if (!empty($this->request->data['Tache']['Tache']) || empty($this->request->data['Recherche']['id'])) {
                $data['Tache'] = array();
                foreach ($this->request->data['Tache']['Tache'] as $ktache => $tache) {
                    $data['Tache']['Tache'][$ktache] = $tache;
                }
            } else {
                $query = "DELETE FROM recherches_taches WHERE recherche_id = " . $this->request->data['Recherche']['id'] . ";";
                $this->Recherche->query($query);
            }
            // Métadonnées
            if (!empty($this->request->data['Metadonnee']['Metadonnee']) || empty($this->request->data['Recherche']['id'])) {
                foreach ($this->request->data['Metadonnee']['Metadonnee'] as $kmetadonnee => $metadonnee) {
                    $data['Metadonnee']['Metadonnee'][$kmetadonnee] = $metadonnee;
                }
            } else {
                $query = "DELETE FROM recherches_metadonnees WHERE recherche_id = " . $this->request->data['Recherche']['id'] . ";";
                $this->Recherche->query($query);
            }
            // Valeur par Métadonnées
            if (!empty($this->request->data['Selectvaluemetadonnee']['Selectvaluemetadonnee'])) {
                foreach ($this->request->data['Selectvaluemetadonnee']['Selectvaluemetadonnee'] as $typemetadonneeid => $valueMetadonnee) {
                    $data['Selectvaluemetadonnee']['Selectvaluemetadonnee'][$typemetadonneeid] = $valueMetadonnee;
                }
            }

            if (!empty($this->request->data['Desktop']['Desktop'])) {
                foreach ($this->request->data['Desktop']['Desktop'] as $kdesktop => $desktop) {
                    $data['Desktop']['Desktop'][$kdesktop] = $desktop;
                }
            }

            if (!empty($this->request->data['Organisme']['Organisme'])) {
                $data['Organisme'] = array();
                foreach ($this->request->data['Organisme']['Organisme'] as $korganisme => $organisme) {
                    $data['Organisme']['Organisme'][$korganisme] = $organisme;
                }
            }

            if (!empty($this->request->data['Service']['Service'])) {
                $data['Service'] = array();
                foreach ($this->request->data['Service']['Service'] as $kservice => $service) {
                    $data['Service']['Service'][$kservice] = $service;
                }
            }
            $data['Recherche']['user_id'] = $this->Session->read('Auth.User.id');
            $data['Recherche']['desktop_id'] = $this->Session->read('Auth.User.Desktop.id');

            $this->Recherche->begin();
            $success = $this->Recherche->saveAll($data);
            $metasList = $this->Recherche->Metadonnee->find(
                    'all', array(
                'recusive' => -1,
                'contain' => false
                    )
            );

            $typemetadonneeIds = Hash::extract($metasList, '{n}.Metadonnee.typemetadonnee_id');
            $valOfSelect = Configure::read('Selectvaluemetadonnee.id');

            // on regarde les anciens enregistrements entre le circut et les sous-types
            /* $records = $this->Recherche->RechercheSelectvaluemetadonnee->find(
              'list',
              array(
              'fields' => array(
              "RechercheSelectvaluemetadonnee.id",
              "RechercheSelectvaluemetadonnee.metadonnee_id"
              ),
              'conditions' => array(
              "RechercheSelectvaluemetadonnee.recherche_id" => $id
              )
              )
              );
              debug($records);

              $oldrecordsids = array_keys( $records );
              debug($oldrecordsids);
              $nouveauxids = Hash::filter( (array)Hash::extract( $data, 'Selectvaluemetadonnee.Selectvaluemetadonnee.{n}' ) );
              $idsenmoins = array_diff( $oldrecordsids, $nouveauxids );
              if( !empty( $idsenmoins ) && !empty( $id ) ) {
              $success = $this->Recherche->RechercheSelectvaluemetadonnee->deleteAll(
              array(
              "RechercheSelectvaluemetadonnee.recherche_id" => $id,
              "RechercheSelectvaluemetadonnee.metadonnee_id" => $idsenmoins
              )
              ) && $success;
              }
              debug($idsenmoins);

              // En plus -> Ajouter
              $idsenplus = array_diff( $nouveauxids, $oldrecordsids );
              debug($idsenplus);
              die();
              if( !empty( $idsenplus ) ) {
              foreach( $idsenplus as $idenplus ) {
              $record = array(
              'RechercheSelectvaluemetadonnee' => array(
              "recherche_id" => $id,
              "metadonnee_id" => $idenplus
              )
              );

              $this->Recherche->RechercheSelectvaluemetadonnee->create( $record );
              $success = $this->Recherche->RechercheSelectvaluemetadonnee->save() && $success;
              }
              } */

            // on regarde les nouveautés
//                    $nouveauxids = Hash::extract( $this->request->data, "Soustype.Soustype.{n}" );
            // si des différences existent on note les ids enlevés, les liens coupés entre circuit et sous-types
//                    $idsenmoins = array_diff( $oldrecordsids, $nouveauxids );

            foreach ($data['Selectvaluemetadonnee']['Selectvaluemetadonnee'] as $meta_id => $selectValueMeta) {
                $metadon = $this->Recherche->Metadonnee->find('first', array('conditions' => array('Metadonnee.id' => $meta_id), 'recursive' => -1));

                if ($metadon['Metadonnee']['typemetadonnee_id'] != $valOfSelect) {
                    $metaValue = array(
                        'RechercheSelectvaluemetadonnee' => array(
                            'recherche_id' => $this->Recherche->id,
                            'selectvaluemetadonnee_id' => null,
                            'metadonnee_id' => $metadon['Metadonnee']['id'],
                            'valeur' => $selectValueMeta
                        )
                    );

                    $this->Recherche->RechercheSelectvaluemetadonnee->create($metaValue);
                    $success = $this->Recherche->RechercheSelectvaluemetadonnee->save() && $success;
                } else {
                    $metaValue = array(
                        'RechercheSelectvaluemetadonnee' => array(
                            'recherche_id' => $this->Recherche->id,
                            'selectvaluemetadonnee_id' => $selectValueMeta,
                            'metadonnee_id' => $metadon['Metadonnee']['id'],
                            'valeur' => null
                        )
                    );

                    // Comparaison des ids avant/apres pour mise à jour
                    // on regarde les anciens enregistrements entre la recherche et les valeurs de métadonnées
                    /* $records = $this->Recherche->RechercheSelectvaluemetadonnee->find(
                      'list',
                      array(
                      'fields' => array( "RechercheSelectvaluemetadonnee.id", "RechercheSelectvaluemetadonnee.selectvaluemetadonnee_id" ),
                      'conditions' => array(
                      "RechercheSelectvaluemetadonnee.recherche_id" => $id
                      )
                      )
                      ); */

                    if (empty($id)) {
                        $this->Recherche->RechercheSelectvaluemetadonnee->create($metaValue);
                        $success = $this->Recherche->RechercheSelectvaluemetadonnee->save() && $success;
                    }
                }
            }

            if ($success) {
                $this->Recherche->commit();
                $this->Jsonmsg->valid();
            } else {
                $this->Recherche->rollback();
            }
            $this->Jsonmsg->send();
        } else {
            $mode = "";
            if ($id != null) {
                if ($id == 0) {
                    $mode = "add";
                } else {
                    $mode = "edit";
                    if ($forcemode != null && $forcemode == 'search') {
                        $mode = 'search';
                        $this->set('recherches', $this->Recherche->find('list', array('conditions' => array('Recherche.user_id' => $this->Session->read('Auth.User.id')))));
                    }
                    $this->request->data = $this->Recherche->find(
                            'first', array(
                        'conditions' => array(
                            'Recherche.id' => $id
                        ),
                        'contain' => array(
                            'Desktop',
                            'Metadonnee',
                            'RechercheSelectvaluemetadonnee',
                            'Contact',
                            'Organisme',
                            'Type',
                            'Soustype',
                            'Dossier',
                            'Affaire',
                            'Tache',
                            'Circuit'
                        )
                            )
                    );
                    $metaToDisplay = array();
                    foreach ($this->request->data['RechercheSelectvaluemetadonnee'] as $i => $valueSelect) {
                        $key = $valueSelect['metadonnee_id'];
                        if ($valueSelect['selectvaluemetadonnee_id'] === null && $valueSelect['valeur'] === null) {
                            $metaToDisplay[$key] = null;
                        } else if ($valueSelect['selectvaluemetadonnee_id'] === null) {
                            $metaToDisplay[$key] = $valueSelect['valeur'];
                        } else {
                            $metaToDisplay[$key] = $valueSelect['selectvaluemetadonnee_id'];
                        }
                    }
                    $this->request->data['Selectvaluemetadonnee']['Selectvaluemetadonnee'] = $metaToDisplay;
                }
            } else {
                $mode = "search";
                $this->set('recherches', $this->Recherche->find('list', array('conditions' => array('Recherche.user_id' => $this->Session->read('Auth.User.id')))));
            }
            $this->set('mode', $mode);
            $this->_setOptions();
        }
    }

    /**
     * Récupération de la liste des recherches
     *
     * @logical-group Recherches
     * @user-profile User
     *
     * @access public
     * @return void
     */
    public function getRecherches() {

        $this->Recherche->recursive = -1;
        $recherches = $this->Recherche->find('all', array('conditions' => array('Recherche.user_id' => $this->Session->read('Auth.User.id'))));
        for ($i = 0; $i < count($recherches); $i++) {
            $recherches[$i]['right_view'] = true;
            $recherches[$i]['right_edit'] = true;
            $recherches[$i]['right_delete'] = true;
        }
        $this->set('recherches', $recherches);
    }

    /**
     * Suppression d'une recherche enregistrée
     *
     * @logical-group Recherches
     * @user-profile User
     *
     * @access public
     * @param integer $id identifiant de la recherche enregistrée
     * @return void
     */
    public function delete($id = null) {
        $this->Jsonmsg->init(__d('default', 'delete.error'));
        $recherche = $this->Recherche->find('first', array('conditions' => array('Recherche.id' => $id)));
        $this->Recherche->begin();
        if ($this->Recherche->delete($id)) {
            $this->Recherche->commit();
            $this->Jsonmsg->valid(__d('default', 'delete.ok'));
        } else {
            $this->Recherche->rollback();
        }
        $this->Jsonmsg->send();
    }

    /**
     * Lancement de la recherche
     *
     * @logical-group Recherches
     * @user-profile User
     *
     * @access public
     * @param integer $id identifiant de la recherche
     * @return void
     */
    public function recherche($id = null) {
//    $this->Recherche->recursive = -1;
        if ($id != null) {
            $this->request->data = $this->Recherche->find('first', array('conditions' => array('Recherche.id' => $id)));
        }
        $params = $this->request->data;

        if (!empty($params)) {
// debug($params);
            if (isset($params['tri'])) {
                $tri = $params['tri'];
            }
            if (!empty($this->request->data['Recherche']['contactdept'])) {
                //Pour les régions
                $dept = ClassRegistry::init('Ban')->find(
                        'first', array(
                    'fields' => array('Ban.name', 'Ban.id'),
                    'conditions' => array(
                        'Ban.id' => $this->request->data['Recherche']['contactdept']
                    ),
                    'recursive' => -1,
                    'contain' => false
                        )
                );
//                $params['Recherche']['contactdept'] = $dept['Ban']['name'];
                $params['Recherche']['contactdept'] = $dept['Ban']['id'];
                // Pour les communes
                $commune = ClassRegistry::init('Ban')->Bancommune->find(
                        'first', array(
                    'fields' => array('Bancommune.name', 'Bancommune.id'),
                    'conditions' => array(
                        'Bancommune.id' => $this->request->data['Recherche']['contactcommune']
                    ),
                    'recursive' => -1,
                    'contain' => false
                        )
                );
//                $params['Recherche']['contactcommune'] = $commune['Bancommune']['name'];
                $params['Recherche']['contactcommune'] = $commune['Bancommune']['id'];
            }

            if (!Configure::read('Conf.SAERP')) {
                $datasAuthorizedType = $this->DataAuthorized->getAuthTypes(array('withSoustypes' => true));
                foreach ($datasAuthorizedType as $t => $type) {
                    $sousTypeIds[$type['Type']['id']] = Hash::extract($type, 'Soustype.{n}.id');
                }
            } else {
                $sousTypeIds = array();
            }
            //on lance la recherche
            $flux_tmp = array();
            $querydata = $this->Recherche->search($params, $id, $sousTypeIds);

            // On prépare les conditions en fonction des droits
            $flux = array();
            $this->loadModel('Desktop');
            $desktopId = $this->Session->read('Auth.User.Courrier.Desktop.id');
            $daros = $this->Desktop->getDaros($desktopId);
            $soustypes = $this->Recherche->Soustype->find('list', array('contain' => false));

            $allowedSoustypes = array();
            if (Configure::read('Use.AuthRights') && !Configure::read('Conf.SAERP')) {
                $allowedSoustypes = $this->DataAuthorized->getAuthSoustypes(array('find' => 'all'));
            } else {
                $allowedSoustypes = $soustypes;
            }
//			foreach ($soustypes as $kSoustype => $soustype) {
//				$daco = $this->DataAcl->Daco->find('first', array('conditions' => array('Daco.model' => 'Soustype', 'Daco.foreign_key' => $kSoustype), 'contain' => false));
//				$chks = array();
//				foreach ($daros as $daro) {
//					$chks[] = $this->DataAcl->check($daro['Daro'], $daco['Daco']);
//				}
//				if (!in_array(false, $chks, true)) {
//					$allowedSoustypes[$kSoustype] = $soustype;
//				}
//			}
//debug( $this->DataAuthorized->getAuthSoustypes(array('find' => 'all')));
            // Si aucune condition, on ne retourne rien
            if (!empty($querydata)) {
                $querydata['limit'] = 10;
                if (isset($tri) && !empty($tri)) {
                    $querydata['order'] = $tri;
                } else {
                    $querydata['order'] = 'Courrier.datereception DESC';
                }
                // Aucune condition sur les utilisateurs du courrier si Admin
                $bureaux = Hash::extract($this->Session->read('Auth.User.SecondaryDesktops'), '{n}.Group.name');
                if ($this->Session->read('Auth.User.Desktop.Group.name') != 'Admin' && !in_array('Admin', $bureaux)) {
//                if( $this->Session->read('Auth.User.Desktop.Group.name') != 'Admin' ) {
                    $querydata['conditions'][] = array(
                        'OR' => array(
                            'OR' => array(
                                array(
                                    'Courrier.soustype_id' => array_keys($allowedSoustypes),
                                    'Courrier.soustype_id IS NOT NULL'
                                ),
                                array(
                                    'Courrier.soustype_id IS NULL'
                                )
                            ),
                            'Soustype.id' => array_keys($allowedSoustypes),
                            'Courrier.user_creator_id' => $this->Session->read('Auth.User.id'),
                            'Courrier.desktop_creator_id' => $desktopId
                        )
                    );
                }

                $this->paginate = array('Courrier' => $querydata);

                $flux_tmp = $this->paginate('Courrier');
            }
//debug($querydata);
            foreach ($flux_tmp as $kflx => $flx) {
//				if (array_key_exists($flx['Soustype']['id'], $allowedSoustypes) || $flx['Courrier']['user_creator_id'] == $this->Session->read('Auth.User.id')) {
                $flx['retard'] = !$this->Courrier->isInTime($flx['Courrier']['id']);
                $flx['right_read'] = true;
                $flx['right_update'] = true;
//					$flx['right_delete'] = true;
                $flux[] = $flx;
//				}
            }
            $this->set(
                    'etat', array(
                '-1' => 'Refusé',
                '0' => 'Validé',
                '1' => 'En cours de traitement',
                '2' => 'Clôturé'
                    )
            );

            $this->set('flux', $flux);
            if (isset($tri) && $tri != '') {
                list($colone, $order) = explode(' ', $tri);

                switch ($colone) {
                    case 'Courrier.reference':
                        $ancien_name = 'reference';
                        break;
                    case 'Courrier.name':
                        $ancien_name = 'nom';
                        break;
                    case 'Courrier.objet':
                        $ancien_name = 'objet';
                        break;
                    case 'Organisme.name':
                        $ancien_name = 'expediteur';
                        break;
                    case 'Courrier.datereception':
                        $ancien_name = 'datereception';
                        break;
                    case 'Courrier.mail_retard_envoye':
                        $ancien_name = 'retard';
                        break;
                    default :
                        $ancien_name = '';
                        break;
                }
                $this->set('ancien_order', $order);
                $this->set('ancien_name', $ancien_name);
            }
        }
    }

    /**
     * Export de la recherche au format CSV
     *
     * @logical-group Recherches
     * @user-profile User
     *
     * @access public
     * @return void
     */
    public function exportcsv() {

//        ini_set( 'max_execution_time', 0 );
//        ini_set( 'memory_limit', '2048M' );
        if (!Configure::read('Conf.SAERP')) {
            $datasAuthorizedType = $this->DataAuthorized->getAuthTypes(array('withSoustypes' => true));
            foreach ($datasAuthorizedType as $t => $type) {
                $sousTypeIds[$type['Type']['id']] = Hash::extract($type, 'Soustype.{n}.id');
            }
        } else {
            $sousTypeIds = array();
        }
        $querydata = $this->Recherche->search(Hash::expand($this->request->params['named'], '__'), null, $sousTypeIds);
        unset($querydata['limit']);
//debug($querydata);
//die();
        $querydata['order'] = 'Courrier.datereception ASC';

        $flux = $this->Courrier->find('all', $querydata);
        foreach ($flux as $i => $flx) {

            $flx['Courrier']['retard'] = !$this->Courrier->isInTime($flx['Courrier']['id']);
            $flux[$i]['Courrier'] = $flx['Courrier'];

            if (isset($flx['Metadonnee']['name']) && $flx['Metadonnee']['name'] == 'Montant HT') {
                $flux[$i]['Lettre']['montant'] = $flx['CourrierMetadonnee']['valeur'];
            } else {
                $flux[$i]['Lettre']['montant'] = '';
            }              
            $courriers_metadonnees = $this->Courrier->CourrierMetadonnee->find(
                'all',
                array(
                    'conditions' => array(
                        'CourrierMetadonnee.courrier_id' => $flx['Courrier']['id']
                    ),
                    'contain' => array(
                        'Metadonnee' => array(
                            'Selectvaluemetadonnee'
                        )
                    )
                )
            );
            foreach( $courriers_metadonnees as $c => $valeurs) {
                foreach( $valeurs['Metadonnee']['Selectvaluemetadonnee'] as $s => $valeurChoisi ) {
                    $selected[$valeurChoisi['id']] = $valeurChoisi['name'];
                }
                if( $valeurs['Metadonnee']['typemetadonnee_id'] == 4) {
                    $metas[$c] = $valeurs['Metadonnee']['name']. " : ".$selected[$valeurs['CourrierMetadonnee']['valeur']];
                }
                else {
                    $metas[$c] = $valeurs['Metadonnee']['name']. " : ".$valeurs['CourrierMetadonnee']['valeur'];
                }
            }
            $flux[$i]['Metadonnee']['valeur'] = implode(' // ', $metas);
        }

        $this->layout = '';
        $this->set(
                'etat', array(
            '-1' => 'Refusé',
            '0' => 'Traité à clôturer',
            '1' => 'En cours de traitement',
            '2' => 'Clôturé'
                )
        );
        $this->set('services', $this->Courrier->Service->find('list'));
        $this->set(compact('flux'));
    }

    public function charge_soustypes() {
//        $this->autoRender = false;
        $resultats = array();
        foreach ($this->request->data['TypeChoix'] as $key => $value) {
//            if ($key != 0) {
            $soustypes = $this->Recherche->Soustype->find('list', array('fields' => array('Soustype.id', 'Soustype.name'), 'conditions' => array('Soustype.type_id' => $value)));
            foreach ($soustypes as $key => $value) {
                $resultats[$key] = $value;
            }
//            }
        }
//debug($resultats);
        $this->set('resultats', $resultats);
    }

    /**
     * Export des données pour la SAERP
     *
     * @logical-group Recherches
     * @user-profile User
     *
     * @access public
     * @return void
     */
    public function export() {
        $this->set('ariane', array(
            '<a href="/environnement/userenv/0">' . __d('menu', 'Recherche') . '</a>',
            __d('menu', 'export', true)
        ));
        if (!empty($this->request->data)) {
            if ($this->request->data['Recherche']['ctypedocument'] == 'PLI') {
                $queryDataPli = $this->Pliconsultatif->search($this->request->data);
                $queryDataPli['order'] = 'Operation.name ASC';
                $flux = $this->Pliconsultatif->find('all', $queryDataPli);
            } else if ($this->request->data['Recherche']['ctypedocument'] == 'OS') {
                $queryDataOS = $this->Ordreservice->search($this->request->data);
                $queryDataOS['order'] = 'Operation.name ASC';
                $this->paginate = $queryDataOS;
                $flux = $this->paginate($this->Ordreservice);
            } else {
                $querydata = $this->Recherche->search($this->request->data, null, array());
                $querydata['order'] = 'Operation.name ASC';
                $this->paginate = array('Courrier' => $querydata);
                $flux = $this->paginate('Courrier');
            }
            foreach ($flux as $i => $flx) {

                $associations = $this->Courrier->Soustype->Type->Operation->Intituleagentbyoperation->find(
                        'all', array(
                    'conditions' => array(
                        'Intituleagentbyoperation.operation_id' => $flx['Operation']['id']
                    ),
                    'contain' => array(
                        'Desktopmanager' => array(
                            'Desktop' => array(
                                'User',
                                'SecondaryUser'
                            )
                        ),
                        'Intituleagent' => array(
                            'conditions' => array(
                                'Intituleagent.name' => array('RO', 'AO', 'GEST')
                            ),
                            'order' => array('Intituleagent.name ASC')
                        )
                    )
                        )
                );

                if (!empty($associations[0]['Desktopmanager']['Desktop'][0]['User'])) {
                    $flux[$i]['Operation']['ao'] = @$associations[0]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'] . ' ' . @$associations[0]['Desktopmanager']['Desktop'][0]['User'][0]['nom'];
                    $flux[$i]['Operation']['gest'] = @$associations[1]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'] . ' ' . @$associations[1]['Desktopmanager']['Desktop'][0]['User'][0]['nom'];
                    $flux[$i]['Operation']['ro'] = @$associations[2]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'] . ' ' . @$associations[2]['Desktopmanager']['Desktop'][0]['User'][0]['nom'];
                } else {
                    $flux[$i]['Operation']['ao'] = @$associations[0]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'] . ' ' . @$associations[0]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['nom'];
                    $flux[$i]['Operation']['gest'] = @$associations[1]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'] . ' ' . @$associations[1]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['nom'];
                    $flux[$i]['Operation']['ro'] = @$associations[2]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'] . ' ' . @$associations[2]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['nom'];
                }
                if (isset($flx['Metadonnee']['name']) && $flx['Metadonnee']['name'] == 'Montant HT') {
                    $flux[$i]['Lettre']['montant'] = $flx['CourrierMetadonnee']['valeur'];
                }
                $flx['Courrier']['retard'] = !$this->Courrier->isInTime($flx['Courrier']['id']);
                
                // destinataire du flux
//                $civilite = array(
//                    'Monsieur' => 'Monsieur',
//                    'Madame' => 'Madame', 
//                    'Monsieur et Madame' => 'Monsieur et Madame'
//                );
//                $flx['Destinataire']['name'] = @$flx['Organisme']['name']. ' / '.$civilite[$flx['Contact']['civilite']].' '.$flx['Contact']['name'];
//                
//                $flux[$i]['Destinataire']['name'] = $flx['Destinataire']['name'];
//                $flux[$i]['Destinataire']['numvoie'] = $flx['Contact']['numvoie'];
//                $flux[$i]['Destinataire']['nomvoie'] = $flx['Contact']['nomvoie'];
//                $flux[$i]['Destinataire']['compl'] = $flx['Contact']['compl'];
//                $flux[$i]['Destinataire']['cp'] = $flx['Contact']['cp'];
//                $flux[$i]['Destinataire']['ville'] = $flx['Contact']['ville'];
                    
                $flux[$i]['Courrier'] = $flx['Courrier'];
                $flux[$i]['right_read'] = true;
                $flux[$i]['right_view'] = true;
            }

            $countFlux = count($flux);
            $this->set(compact('flux', 'countFlux'));
        }

        $this->_setOptions();
    }

    /**
     * Export des LN pour la SAERP
     *
     * @logical-group Recherches
     * @user-profile User
     *
     * @access public
     * @return void
     */
    public function exportLettre() {
        $this->set('ariane', array(
            '<a href="/environnement/userenv/0">' . __d('menu', 'Recherche') . '</a>',
            __d('menu', 'export', true)
        ));

//debug($this->request->params['named']);
//die();
        if (isset($this->request->params['named']['Recherche__ctypedocument']) && $this->request->params['named']['Recherche__ctypedocument'] == 'OS') {
            $querydata = $this->Ordreservice->search(Hash::expand($this->request->params['named'], '__'));
            unset($querydata['limit']);
            $querydata['order'] = 'Operation.name ASC';
//            $querydata['group'] = array('Operation.name', 'Ordreservice.id', 'Courrier.id', 'Marche.id', 'Operation.id');
//            foreach ($querydata['joins'] as $j => $info) {
//                if ($info['alias'] == 'CourrierMetadonnee') {
//                    $querydata['group'] = array_merge(
//                            $querydata['group'], array(
//                        'CourrierMetaodnnee.id', 'Metadonnee.id'
//                            )
//                    );
//                }
//            }
            $flux = $this->Ordreservice->find('all', $querydata);
        } else if (isset($this->request->params['named']['Recherche__ctypedocument']) && $this->request->params['named']['Recherche__ctypedocument'] == 'PLI') {
            $querydata = $this->Pliconsultatif->search(Hash::expand($this->request->params['named'], '__'));
            unset($querydata['limit']);
            $querydata['order'] = 'Operation.name ASC';
//            $querydata['group'] = array('Operation.name', 'Pliconsultatif.id', 'Origineflux.id', 'Courrier.id', 'Consultation.id', 'Operation.id', 'CourrierMetadonnee.id', 'Metadonnee.id');
            $flux = $this->Pliconsultatif->find('all', $querydata);
        } else {
            $querydata = $this->Recherche->search(Hash::expand($this->request->params['named'], '__'), null, array());
            unset($querydata['limit']);
            $querydata['order'] = 'Operation.name ASC';
            $flux = $this->Courrier->find('all', $querydata);
        }
        foreach ($flux as $i => $flx) {
            $associations = $this->Courrier->Soustype->Type->Operation->Intituleagentbyoperation->find(
                    'all', array(
                'conditions' => array(
                    'Intituleagentbyoperation.operation_id' => $flx['Operation']['id']
                ),
                'contain' => array(
                    'Desktopmanager' => array(
                        'Desktop' => array(
                            'User',
                            'SecondaryUser'
                        )
                    ),
                    'Intituleagent' => array(
                        'conditions' => array(
                            'Intituleagent.name' => array('RO', 'AO', 'GEST')
                        ),
                        'order' => array('Intituleagent.name ASC')
                    )
                )
                    )
            ); 
            if (!empty($associations[0]['Desktopmanager']['Desktop'][0]['User'])) {
                $flux[$i]['Operation']['ao'] = isset( $associations[0]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'] ) ? ($associations[0]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'].' '.@$associations[0]['Desktopmanager']['Desktop'][0]['User'][0]['nom'] ) : ($associations[0]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'].' '.@$associations[0]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['nom']);
                $flux[$i]['Operation']['gest'] = isset( $associations[1]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'] ) ? ($associations[1]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'].' '.@$associations[1]['Desktopmanager']['Desktop'][0]['User'][0]['nom'] ) : ($associations[1]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'].' '.@$associations[1]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['nom']);
                $flux[$i]['Operation']['ro'] = isset( $associations[2]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'] ) ? ($associations[2]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'].' '.@$associations[2]['Desktopmanager']['Desktop'][0]['User'][0]['nom'] ) : ($associations[2]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'].' '.@$associations[2]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['nom']);
            } else {
                $flux[$i]['Operation']['ao'] = isset( $associations[0]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'] ) ? ($associations[0]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'].' '.@$associations[0]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['nom'] ) : ($associations[0]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'].' '.@$associations[0]['Desktopmanager']['Desktop'][0]['User'][0]['nom']);
                $flux[$i]['Operation']['gest'] = isset( $associations[1]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'] ) ? ($associations[1]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'].' '.@$associations[1]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['nom'] ) : ($associations[1]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'].' '.@$associations[1]['Desktopmanager']['Desktop'][0]['User'][0]['nom']);
                $flux[$i]['Operation']['ro'] = isset( $associations[2]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'] ) ? ($associations[2]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'].' '.@$associations[2]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['nom'] ) : ($associations[2]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'].' '.@$associations[2]['Desktopmanager']['Desktop'][0]['User'][0]['nom']);
            } 
            if (isset($flx['Metadonnee']['name']) && $flx['Metadonnee']['name'] == 'Montant HT') {
                $flux[$i]['Lettre']['montant'] = $flx['CourrierMetadonnee']['valeur'];
            } else {
                $flux[$i]['Lettre']['montant'] = '';
            }
            $flx['Courrier']['retard'] = !$this->Courrier->isInTime($flx['Courrier']['id']);
            
             // destinataire du flux
            $civilite = array(
                'Monsieur' => 'Monsieur',
                'Madame' => 'Madame', 
                'Monsieur et Madame' => 'Monsieur et Madame'
            );
            $flx['Destinataire']['name'] = @$flx['Organisme']['name']. ' / '.$civilite[$flx['Contact']['civilite']].' '.$flx['Contact']['name'];

            $flux[$i]['Destinataire']['name'] = $flx['Destinataire']['name'];
            $flux[$i]['Destinataire']['numvoie'] = $flx['Contact']['numvoie'];
            $flux[$i]['Destinataire']['nomvoie'] = $flx['Contact']['nomvoie'];
            $flux[$i]['Destinataire']['compl'] = $flx['Contact']['compl'];
            $flux[$i]['Destinataire']['cp'] = $flx['Contact']['cp'];
            $flux[$i]['Destinataire']['ville'] = $flx['Contact']['ville'];
            
            $flux[$i]['Courrier'] = $flx['Courrier'];
        }

//debug($flux);
//die();

        $this->set(compact('flux'));
        $this->layout = '';

        $this->_setOptions();
    }

}

?>
