<?php

App::import('Vendor', 'nusoap/lib/nusoap');

/**
 * WebService
 *
 * Webservices controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 7
 * @author Stéphane Sampaio
 * @copyright Initié par ADULLACT - Développé par ADULLACT Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		Controller
 */
class WebservicesController extends AppController {

	/**
	 * Controller name
	 *
	 * @access public
	 * @var string
	 */
	public $name = 'Webservices';

	/**
	 * Controller uses
	 *
	 * @access public
	 * @var array
	 */
	public $uses = array();

    
    protected $_lastSequenceDocument = null;
    
	/**
	 * CakePHP callback
	 *
	 * Désactivation de l'autentification par les ACLs
	 * Mise en place un autentification Basic Auth
	 *
	 */
	public function beforeFilter() {
		parent::beforeFilter();
		$this->Auth->loginAction = array(
			'controller' => $this->name,
			'action' => 'chk'
		);
		$this->Auth->authenticate = array(
			'Basic' => array('userModel' => 'User')
		);
		$this->Auth->allow('service');
	}

	/**
	 * Vérification des informations de connexion (Basic Authenticate)
	 *
	 * @logical-group WebService
	 *
	 * @access public
	 * @return void
	 */
	public function chk() {
		$data = array(
			'username' => $_SERVER['PHP_AUTH_USER'],
			'password' => $this->Auth->password($_SERVER['PHP_AUTH_PW'])
		);
		if ($this->Auth->login($data)) {
			$this->_logDebug('login ok');
		} else {
			$this->_logDebug('login error');
		}
	}

	/**
	 * Point d'entrée du webservice
	 *
	 * @logical-group WebService
	 *
	 * @access public
	 * @return void
	 */
	public function service() {
		Configure::write('debug', 2);
		ini_set("soap.wsdl_cache_enabled", "0"); // disabling WSDL cache
		$server = new SoapServer('http://' . $_SERVER['HTTP_HOST'] . '/files/wsdl/webgfc.wsdl');
		$server->setObject($this);
		if ($_SERVER['REQUEST_METHOD'] == 'POST') {
			$this->autoRender = false;
			Configure::write('debug', 0);
			ob_start();
			$server->handle();
			$out = ob_get_clean();
			$this->response->type('text/xml');
			$this->response->body($out);
		} else {
			echo 'GET requests are not allowed, please use POST requests.';
		}
	}

	/**
	 * Test du webservice
	 *
	 * @logical-group WebService
	 *
	 * @access public
	 * @return void
	 */
	public function echotest($echoRequest) {
		return $echoRequest . ' ' . env('SERVER_NAME');
	}

	/**
	 * Récupération de la liste des collectivités
	 *
	 * @logical-group WebService
	 *
	 * @access public
	 * @return void
	 */
	public function getGFCCollectivites() {
		$this->loadModel('Collectivite');
		$collectivites = $this->Collectivite->find('list', array('conditions' => array('Collectivite.active' => true)));
		$return = array();
		foreach ($collectivites as $key => $val) {
			$return[] = array($key, $val);
		}
		return $return;
	}

	/**
	 * Définition de la connexion utilisée
	 *
	 * @access protected
	 * @param integer $collectiviteId indentifiant d'une collectivité
	 * @return void
	 */
	protected function _setConnection($collectiviteId) {
		$this->loadModel('Collectivite');
		$coll = $this->Collectivite->find('first', array('conditions' => array('Collectivite.id' => $collectiviteId)));
		Configure::write('conn', $coll['Collectivite']['conn']);
	}

	/**
	 * Récupération de la liste des types
	 *
	 * @logical-group WebService
	 *
	 * @access public
	 * @param integer $collectiviteId identifiant d'une collectivité
	 * @return array
	 */
	public function getGFCTypes($collectiviteId) {
		$this->_setConnection($collectiviteId);
		$this->loadModel('Type');
		$this->Type->setDataSource(Configure::read('conn'));
		$types = $this->Type->find('list');
		$return = array();
		foreach ($types as $key => $val) {
			$return[] = array($key, $val);
		}
		return $return;
	}

	/**
	 * Récupération de la liste des types
	 *
	 * @logical-group WebService
	 *
	 * @access public
	 * @param integer $collectiviteId identifiant d'une collectivité
	 * @param integer $typeId identifiant d'un type
	 * @return array
	 */
	public function getGFCSoustypes($collectiviteId, $typeId) {
		$this->_setConnection($collectiviteId);
		$this->loadModel('Soustype');
		$this->Soustype->setDataSource(Configure::read('conn'));
		$soustypes = $this->Soustype->find('list', array('conditions' => array('Soustype.type_id' => $typeId)));
		$return = array();
		foreach ($soustypes as $key => $val) {
			$return[] = array($key, $val);
		}
		return $return;
	}

	/**
	 * Création d'un flux
	 *
	 * @logical-group WebService
	 *
	 * @deprecated since version 0.9
	 *
	 * @param integer $collectiviteId identifiant d'une collectivité
	 * @param integer $soustypeId identifiant d'un sous-type
	 * @param string $contactNom nom du contact
	 * @param string $titre titre du flux
	 * @param string $objet objet du flux
	 * @param base64 $fichier_associe contenu du document lié
	 * @param string $username identifiant de connexion de l'utilisateur cible
	 * @return array|string
	 */
	public function createCourrier($collectiviteId, $soustypeId, $contactNom, $titre, $objet, $fichier_associe, $username) {
        $this->_setConnection($collectiviteId);
		$return = array(
			'CodeRetour' => 'KO',
			'CourrierId' => -1,
			'Message' => 'Impossible de créer le flux.',
			'Severite' => 'important'
		);
		$this->loadModel('Courrier');
		$this->loadModel('User');
		$this->loadModel('Bancontenu');
		$this->Courrier->setDataSource(Configure::read('conn'));
		$this->Courrier->Soustype->setDataSource(Configure::read('conn'));
		$this->User->setDataSource(Configure::read('conn'));
		$this->User->recursive = -1;
		$this->Bancontenu->setDataSource(Configure::read('conn'));
		$this->Courrier->Metadonnee->setDataSource(Configure::read('conn'));
		$this->Courrier->Soustype->setDataSource(Configure::read('conn'));
		$this->Courrier->Soustype->Type->setDataSource(Configure::read('conn'));
		$querydata = array(
			'conditions' => array(
				'User.username' => $username
			),
			'contain' => array(
                'Desktop' => array(
                    'Group'
                )
				/*$this->User->Desktop->alias => array(
					$this->User->Group->alias
				),
				$this->User->SecondaryDesktop->alias => array(
					$this->User->SecondaryDesktop->Group->alias
				),*/
			)
		);
		$user = $this->User->find('first', $querydata);

        
//        $desktopsmanagers = $this->User->Desktop->getDesktopManager($user['Desktop']['id']);
//Configure::write( 'debug', 2);
//$this->log($user, 'debug');
//die();
//$this->log(array( $collectiviteId, $soustypeId, $contactNom, $titre, $objet, $fichier_associe, $username ), 'debug');
//$this->log($user, 'debug');

		if (empty($user)) {
			$return['CodeRetour'] = "KONOUSER";
			$return['Message'] .= " L'utilisateur n'existe pas.";
		} else {
			$desktopId = null;
			if ($user['Desktop']['group_id'] == INIT_GID || $user['Desktop']['group_id'] == DISP_GID) {
				$desktopId = $user['Desktop']['id'];
			} else {
				$found = false;
				for ($i = 0; $i < count($user['SecondaryDesktop']); $i++) {
					if ($user['SecondaryDesktop'][$i]['group_id'] == DISP_GID && !$found) {
						$desktopId = $user['SecondaryDesktop'][$i]['id'];
						$found = true;
					}
				}
			}
			if (empty($desktopId)) {
				$return['CodeRetour'] = "KONODESKTOP";
				$return['Message'] .= "L'utilisateur n'a pas de rôle Aiguilleur ou Initiateur.";
			} else {
				$data = array(
					'Courrier' => array(
						'name' => $titre,
						'objet' => $objet,
						'user_creator_id' => $user['User']['id'],
						'soustype_id' => !empty($soustypeId) ? $soustypeId : null,
						'date' => date('Y-m-d'),
						'datereception' => date('Y-m-d'),
//						'reference' => $this->Courrier->genRef($soustypeId)
						'reference' => $this->Courrier->genReferenceUnique(),
						'contact_id' => configure::read('Sanscontact.id'),
						'oragnisme_id' => configure::read('Sansorganisme.id')
					)
				);

				$this->Courrier->create($data);
				if ($this->Courrier->save()) {
                    $courrierId = $this->Courrier->id;
					$return['CodeRetour'] = "CREATED";
//					$return['Message'] = "Le flux a été créé";
					$return['Message'] = "";
					$return['Severite'] = "normal";

                    $groupId = $user['Desktop']['group_id'];
//                    foreach( $desktopsmanagers as $i => $desktopmanagerId) {
                    if( $groupId == 7 ) {
                        $this->Bancontenu->add($desktopId, BAN_AIGUILLAGE, $courrierId);
                    }
                    else if( $groupId == 3 ) {
                        $this->Bancontenu->add($desktopId, BAN_DOCS, $courrierId);
                    }
                    else {
                        $return['CodeRetour'] = "ERROR";
                        $return['Message'] .= "L'utilisateur n'a pas de rôle Aiguilleur ou Initiateur.";
                    }

//                    }
                    $return['CourrierId'] = $courrierId;                    

                    //TODO: si typeID not null alors on insérer dnas le circuit
                    
//					$courrier = $this->Courrier->read();
					/*
                     $courrier = $this->Courrier->find(
                        'first',
                        array(
                            'conditions' => array(
                                'Courrier.id' => $courrierId
                            ),
                            'contain' => false
                        )
                    );
					$soustype = $this->Courrier->Soustype->find('first', array('conditions' => array('Soustype.id' => $data['Courrier']['soustype_id']), 'contain' => false));
					
					//ajout dans la bannette du redacteur
					$this->Bancontenu->add($data['Courrier']['user_creator_id'], BAN_DOCS, $data['Courrier']['id']);
					$inserted = false;
					if (!$this->Courrier->Traitement->targetExists($this->Courrier->id)) {
						if ($this->Courrier->Traitement->Circuit->insertDansCircuit($soustype['Soustype']['circuit_id'], $courrier['Courrier']['id'])) {
							$return['CodeRetour'] = "INSERTED";
							$return['Message'] = "Le flux a été créé et inséré dans un circuit";
							//on change l etat du flux dans la bannette du createur
							$this->Bancontenu->end($courrier['Courrier']['user_creator_id'], BAN_DOCS, $courrier['Courrier']['id']);
							//recuperation des prochains utilisateurs concerné par le flux
							$nextUsers = $this->Courrier->Traitement->whoIsNext($courrier['Courrier']['id']);
							foreach ($nextUsers as $nextUserId) { //pour chaque prochain utilisateurs concerné
								//on assigne le flux dans les bannettes des prochains utilisateurs concernés
								$this->Bancontenu->add($nextUserId, BAN_FLUX, $courrier['Courrier']['id']);
								//on envoi une notification aux prochains utilisateurs concernés
//                $message = 'Nouveau flux à traiter: ' . $courrier['Courrier']['name'];
//                $this->Notification->add("Nouveau flux", $courrier['Courrier']['user_creator_id'], $nextUserId, $courrier['Courrier']['id'], $message);
							}
							$inserted = true;
						}
                        else {
                            $this->Bancontenu->add($data['Courrier']['user_creator_id'], BAN_AIGUILLAGE, $data['Courrier']['id']);
                        }
					}
					if (!$inserted) { //si le flux n a pas été inséré dans un circuit
						//notification de nouveau courrier
//            $message = 'Nouveau document: ' . $courrier['Courrier']['name'];
//            $this->Notification->add("Nouveau flux", $courrier['Courrier']['user_creator_id'], $courrier['Courrier']['user_creator_id'], $courrier['Courrier']['id'], $message);
					}*/
					$this->_genFileAssoc($fichier_associe, $courrierId, $return);
				}
			}
		}
		return $return;
	}

	/**
	 * Génération d'un document lié
	 *
	 * @access private
	 * @param base64 $fichier_associe contenu du document lié
	 * @param integer $fluxId identifiant du flux
	 * @param array $return valeur de retour
	 * @return void
	 */
	private function _genFileAssoc($fichier_associe, $fluxId, &$return) {
		$finfo = new finfo(FILEINFO_MIME_TYPE);
		$fileMimeType = $finfo->buffer($fichier_associe);
        $this->loadModel('Document');
//		$filename = "fichier_associe_" . date('Ymd_His');
//        $sql = "SELECT nextval('documents_name_seq'); /*".mktime(true)." "  . $this->_lastSequenceDocument . "*/";

		$fileMimeTypeError = false;
		if ($fileMimeType == "application/pdf") {
			$fileExt = "pdf";
		} else if ($fileMimeType == "application/vnd.oasis.opendocument.text") {
			$fileExt = "odt";
		} else if ($fileMimeType == "image/bmp") {
			$fileExt = "bmp";
		} else if ($fileMimeType == "image/png") {
			$fileExt = "png";
		} else if ($fileMimeType == "image/jpeg") {
			$fileExt = "jpg";
		} else if ($fileMimeType == "image/tiff") {
			$fileExt = "tif";
		} else if ($fileMimeType == "text/plain") {
			$fileExt = "txt";
		} else if ($fileMimeType == "application/vnd.ms-office") {
			$fileExt = "doc";
		} else if ($fileMimeType == "application/msword") {
			$fileExt = "doc";
		} else if ($fileMimeType == "application/rtf") {
			$fileExt = "rtf";
        } else {
			$fileMimeTypeError = true;
		}

        $filename = $this->Document->genNameUnique(). "." . $fileExt;


		if ($fileMimeTypeError) {
			$return['CodeRetour'] .= "FILENOK";
			$return['Message'] .= ". Le fichier n'est pas reconnu";
		} else {
			$doc = array(
				'Document' => array(
					'name' => $filename,
					'size' => strlen($fichier_associe),
					'mime' => $fileMimeType,
					'content' => $fichier_associe, //TODO: prevoir l utilisation d une GED
					'ext' => $fileExt,
					'courrier_id' => $fluxId,
					'main_doc' => true
				)
			);
			$this->Courrier->setDataSource(Configure::read('conn'));
			$this->Courrier->Document->setDataSource(Configure::read('conn'));
			$this->Courrier->Document->create($doc);
			$docsaved = $this->Courrier->Document->save();
			if (empty($docsaved)) {
				$return['CodeRetour'] .= "NOFILE";
				$return['Message'] .= "Le flux a bien été créé mais le fichier n'a pas été envoyé<br />Veuillez ré-essayer.";
                $this->Courrier->delete($fluxId);
			} else {
				$return['CodeRetour'] .= "FILEOK";
				$return['Message'] .= "Le flux a bien été créé et le fichier a été envoyé";
			}
		}
	}

	/**
	 * Fonctionnalité de numérisation de masse (création de flux)
	 *
	 * @logical-group WebService
	 *
	 * @access public
	 * @param integer $collectiviteId identifiant de la collectivité
	 * @param base64 $fichier_associe contenu du fichier lié
	 * @param string $username identifiant de connexion de l'utilisateur cible
	 * @return array
	 */
	public function scanMass($collectiviteId, $fichier_associe, $username) {
		$this->_setConnection($collectiviteId);
		$return = array(
			'CodeRetour' => 'KO',
			'CourrierId' => -1,
			'Message' => 'Impossible de créer le flux.',
			'Severite' => 'important'
		);
		$this->loadModel('Courrier');
		$this->loadModel('User');
		$this->loadModel('Bancontenu');
		$this->Courrier->setDataSource(Configure::read('conn'));
		$this->Courrier->Soustype->setDataSource(Configure::read('conn'));
		$this->Courrier->Soustype->Type->setDataSource(Configure::read('conn'));
		$this->Courrier->Metadonnee->setDataSource(Configure::read('conn'));
		$this->User->setDataSource(Configure::read('conn'));
		$this->Bancontenu->setDataSource(Configure::read('conn'));
		$this->Bancontenu->setDataSource(Configure::read('conn'));
		$this->Bancontenu->Bannette->setDataSource(Configure::read('conn'));
		$querydata = array(
			'conditions' => array(
				'User.username' => $username
			),
			'contain' => array(
				$this->User->Desktop->alias => array(
					$this->User->Group->alias
				),
				$this->User->SecondaryDesktop->alias => array(
					$this->User->SecondaryDesktop->Group->alias
				),
			)
		);
		$user = $this->User->find('first', $querydata);
		if (empty($user)) {
			$return['CodeRetour'] = "KONOUSER";
			$return['Message'] .= " L'utilisateur n'existe pas.";
		} else {
			$desktopId = null;
			if ($user['Desktop']['group_id'] == DISP_GID) {
				$desktopId = $user['Desktop']['id'];
			} else {
				$found = false;
				for ($i = 0; $i < count($user['SecondaryDesktop']); $i++) {
					if ($user['SecondaryDesktop'][$i]['group_id'] == DISP_GID && !$found) {
						$desktopId = $user['SecondaryDesktop'][$i]['id'];
						$found = true;
					}
				}
			}
			if (empty($desktopId)) {
				$return['CodeRetour'] = "KONODESKTOP";
				$return['Message'] .= "L'utilisateur n'a pas de rôle Aiguilleur.";
			} else {
                $reference = "";
                if( empty($reference)) {
                    $reference = $this->Courrier->genReferenceUnique();
                }
				$data = array(
					'Courrier' => array(
						'name' => 'Flux_importé_' . date('Y-m-d_H:i:s'),
                        'reference' => $reference
					)
				);
				$this->Courrier->create($data);
				if ($this->Courrier->save()) {
					$courrierId = $this->Courrier->id;
					$return['CodeRetour'] = "CREATED";
					$return['Message'] = "Le flux a été créé";
					$return['Severite'] = "normal";
					$return['CourrierId'] = $courrierId;
					//ajout dans la bannette de l aiguilleur
					$this->Bancontenu->add($desktopId, BAN_AIGUILLAGE, $courrierId);
//TODO: permettre la notification d un lot complet et non la notification de chaque flux comme ci dessous
//          $name = __d('notification', 'ban_wsnew.name');
//          $description = __d('notification', 'ban_wsnew.description');
//          $type = 'ban_wsnew';
//          $desktops = array(
//            $desktopId => array(
//                $courrierId
//            )
//          );
//          $this->Courrier->Notifieddesktop->Notification->add($name, $description, $type, $desktops);
					$this->_genFileAssoc($fichier_associe, $courrierId, $return);
				}
			}
		}
		return $return;
	}

	/**
	 * Vérification de l'état d'un flux
	 *
	 * @logical-group WebService
	 *
	 * @access public
	 * @param integer $collectiviteId identifiant de la collectivité
	 * @param integer $courrierId identifiant du flux
	 * @return array
	 */
	public function getStatut($collectiviteId, $courrierId) {
		$this->_setConnection($collectiviteId);
		$this->loadModel('Courrier');
		$this->Courrier->setDataSource(Configure::read('conn'));
		$this->Courrier->Soustype->setDataSource(Configure::read('conn'));
		$this->Courrier->Metadonnee->setDataSource(Configure::read('conn'));
		$courrier = $this->Courrier->find('first', array('conditions' => array('Courrier.id' => $courrierId)));
//var_dump($courrier);exit;		
    //valeur de retour par defaut : erreur, le flux n existe pas
		$return = array(
			'CodeRetour' => 'KO',
			'CourrierId' => -1,
			'Message' => 'Le flux n\'existe pas',
			'Severite' => 'normal'
		);
		//si le flux existe
		if (!empty($courrier)) {
			//flux refuse
			$return['CourrierId'] = $courrier['Courrier']['id'];
			if ($this->Courrier->isRefused($courrier['Courrier']['id'])) {
				$return['CodeRetour'] = 'REFUS';
//        $this->loadModel('Notification');
//        $this->Notification->setDataSource(Configure::read('conn'));
//        $notif = $this->Notification->find('first', array('conditions' => array('courrier_id' => $courrier['Courrier']['id'])));
				//on retourne le code REFUS et on donne le Message de la notification de refus
//        if (!empty($notif)) {
//          if (!empty($notif['Notification']['message'])) {
//            $return['Message'] = $notif['Notification']['message'];
//          } else {
//            $return['Message'] = "Aucun message de refus";
//          }
//        }
			} else if ($this->Courrier->isTreated($courrier['Courrier']['id'])) {
				//on retourne le code OK
				$return['CodeRetour'] = 'OK';
				$return['Message'] = 'Le flux a été traité';
			} else if ($this->Courrier->isClosed($courrier['Courrier']['id'])) {
				//on retourne le code OK
				$return['CodeRetour'] = 'TREATED';
				$return['Message'] = 'Le flux est clôturé';
			}
            else {
				//on retourne le code ENCOURS
				$return['CodeRetour'] = 'ENCOURS';
				$return['Message'] = 'Le flux est en cours de traitement';
			}
		}
		return $return;
	}
    
    
    /**
	 * Génération d'un document lié à partir d'un mail
	 *
	 * @access private
	 * @param base64 $fichier_associe contenu du document lié
	 * @param integer $fluxId identifiant du flux
	 * @param array $return valeur de retour
	 * @return void
	 */
	private function _genFileFromMail($fichier_associe, $fluxId, &$return) {
		$finfo = new finfo(FILEINFO_MIME_TYPE);
		$fileMimeType = $finfo->buffer($fichier_associe);
        $this->loadModel('Document');

		$fileMimeTypeError = false;
		if ($fileMimeType == "application/pdf") {
			$fileExt = "eml";
		} else {
			$fileMimeTypeError = true;
		}


		if ($fileMimeTypeError) {
			$return['CodeRetour'] .= "FILENOK";
			$return['Message'] .= ". Le fichier n'est pas reconnu";
		} else {
			$doc = array(
				'Document' => array(
					'name' => $this->Document->genNameUnique(),
					'size' => strlen($fichier_associe),
					'mime' => $fileMimeType,
					'content' => $fichier_associe, //TODO: prevoir l utilisation d une GED
					'ext' => $fileExt,
					'courrier_id' => $fluxId,
					'main_doc' => true
				)
			);
			$this->Courrier->setDataSource(Configure::read('conn'));
			$this->Courrier->Document->setDataSource(Configure::read('conn'));
			$this->Courrier->Document->create($doc);
			$docsaved = $this->Courrier->Document->save();
			if (empty($docsaved)) {
				$return['CodeRetour'] .= "NOFILE";
				$return['Message'] .= ".Le fichier n'a pas été envoyé";
			} else {
				$return['CodeRetour'] .= "FILEOK";
				$return['Message'] .= ". Le fichier a été envoyé";
			}
		}
	}

}

?>
