<?php


/**
 * Consultation model class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 7
 * @author Arnaud Auzolat
 * @copyright Initié par ADULLACT - Développé par ADULLACT Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		app.Model
 */
class Consultation extends AppModel {

	/**
	 *
	 * @var type
	 */
	public $name = 'Consultation';

	/**
	 *
	 * @var type
	 */
	public $useTable = 'consultations';
    
    
    /**
	 * hasMany
	 *
	 * @access public
	 */
	/**
	 * belongsTo
	 *
	 * @access public
	 */
    
    public $belongsTo = array(
		'Operation' => array(
			'className' => 'Operation',
			'foreignKey' => 'operation_id',
			'type' => 'INNER',
			'conditions' => null,
			'fields' => null,
			'order' => null
		)
	);
    
    
    /**
     *  hasMany
     * 
     *  @access public
     */
    public $hasMany = array(
		'Pliconsultatif' => array(
			'className' => 'Pliconsultatif',
			'foreignKey' => 'consultation_id',
			'dependent' => false,
			'conditions' => null,
			'fields' => null,
			'order' => null,
			'limit' => null,
			'offset' => null,
			'exclusive' => null,
			'finderQuery' => null,
			'counterQuery' => null
		),
		'Courrier' => array(
			'className' => 'Courrier',
			'foreignKey' => 'consultation_id',
			'dependent' => false,
			'conditions' => null,
			'fields' => null,
			'order' => null,
			'limit' => null,
			'offset' => null,
			'exclusive' => null,
			'finderQuery' => null,
			'counterQuery' => null
		)
	);
    
    public $validate = array(
		'name' => array(
			array(
				'rule' => array('notBlank'),
				'allowEmpty' => false,
			)

		),
		'numero' => array(
			array(
				'rule' => array('notBlank'),
				'allowEmpty' => false,
			)

		),
		'operation_id' => array(
			array(
				'rule' => array('notBlank'),
				'allowEmpty' => false,
			)

		),
		'datelimite' => array(
			array(
				'rule' => array('notBlank'),
				'allowEmpty' => false,
			)
		),
		'heurelimite' => array(
			array(
				'rule' => array('notBlank'),
				'allowEmpty' => false,
			)

		)
    );

    
    
    /**
     * Fonction permettant de rechercher les marchés
     *
     * @param type $criteres
     * @return array()
     */
    public function search( $criteres ) {

//debug($criteres);
        $conditions = array();
        if( isset( $criteres['SearchConsultation']['name'] ) && !empty( $criteres['SearchConsultation']['name'] ) ) {
            $conditions[] = array( 'Consultation.id' => $criteres['SearchConsultation']['name'] );
        }
        
        if( isset( $criteres['SearchConsultation']['numero'] ) && !empty( $criteres['SearchConsultation']['numero'] ) ) {
            $conditions[] = 'Consultation.numero ILIKE \''.$this->wildcard( '%'.$criteres['SearchConsultation']['numero'].'%' ).'\'';
        }
        
        if( isset( $criteres['SearchConsultation']['operation_id'] ) && !empty( $criteres['SearchConsultation']['operation_id'] ) ) {
            $conditions[] = array( 'Consultation.operation_id' => $criteres['SearchConsultation']['operation_id'] );
        }
        
        // 
        if ( isset( $criteres['SearchConsultation']['datelimite'] ) && !empty( $criteres['SearchConsultation']['datelimite'] ) ) {
            $conditions[] = array('Consultation.datelimite' => $criteres['SearchConsultation']['datelimite']);
		}
            
        if( isset($criteres['SearchConsultation']['active']) ) {
            $active = @$criteres['SearchConsultation']['active'];
            $conditions[] = array( 
                'Consultation.active' => $active
            );
        }
        else {
            $conditions[] = array( 
                'Consultation.active' => true
            );
        }
        
        // Partie pour les OPs
        if( isset($criteres['SearchConsultation']['operationactive']) ) {
            $operationactive = @$criteres['SearchConsultation']['operationactive'];
            $conditions[] = array( 
                'Operation.active' => $operationactive
            );
        }
        else {
            $conditions[] = array( 
                'Operation.active' => true
            );
        }

        $query = array(
            'contain' => array(
                'Pliconsultatif',
                'Operation'
            ),
            'limit' => 20,
            'order' => 'Consultation.name ASC',
            'conditions' => array(
                $conditions
            )
        );
        
//debug($query);
        return $query;
    }
}

?>