<?php

App::uses('AppModel', 'Model');

/**
 * Document model class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 7
 * @author Stéphane Sampaio
 * @copyright Initié par ADULLACT - Développé par ADULLACT Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		app.Model
 */
class Document extends AppModel {

    /**
     * Display field
     *
     * @var string
     */
    public $displayField = 'name';
    protected $_lastSequenceDocument = null;

    /**
     *
     * @var type
     */
    public $actsAs = array('Linkeddocs');

    /**
     *
     * @var type
     */
    public $convertibleMimeTypes = array(
        "application/vnd.oasis.opendocument.text",
        "application/vnd.oasis.opendocument.spreadsheet",
        "image/bmp",
        "image/tiff",
        "text/plain",
        "application/msword",
        "application/x-octetstream",
        "application/msexcel",
        "application/xls",
        "application/vnd.ms-office",
        "application/vnd.ms-excel",
        "application/rtf",
        "application/binary",
        "OCTET-STREAM"
    );

    /**
     *
     * @var type
     */
    public $otherMimeTypes = array(
        'audio' => array(
            'audio/x-wav' => 'wav',
            'audio/mpeg' => 'mp3',
            'application/ogg' => 'ogg',
            'audio/ogg' => 'ogg'
        ),
        'video' => array(
            'video/x-msvideo' => 'avi',
            'video/mpeg' => 'mpg',
            'video/mp4' => 'mp4',
            'application/ogg' => 'ogg',
            'video/ogg' => 'ogg'
        ),
        'image' => array(
            'image/gif' => 'gif',
            'image/jpeg' => 'jpg',
            'image/png' => 'png'
        ),
        'file' => array(
            'application/vnd.oasis.opendocument.text' => 'odt',
            'OCTET-STREAM' => 'odt',
            'application/vnd.oasis.opendocument.spreadsheet' => 'ods',
            'text/plain' => 'txt',
        )
    );

    /**
     * Validation rules
     *
     * @var array
     */
    public $validate = array(
        'name' => array(
            'notBlank' => array(
                'rule' => array('notBlank'),
            //'message' => 'Your custom message here',
            //'allowEmpty' => false,
            //'required' => false,
            //'last' => false, // Stop validation after this rule
            //'on' => 'create', // Limit validation to 'create' or 'update' operations
            ),
        ),
        'size' => array(
            'numeric' => array(
                'rule' => array('numeric'),
            //'message' => 'Your custom message here',
            //'allowEmpty' => false,
            //'required' => false,
            //'last' => false, // Stop validation after this rule
            //'on' => 'create', // Limit validation to 'create' or 'update' operations
            ),
        ),
        'courrier_id' => array(
            'numeric' => array(
                'rule' => array('numeric'),
            //'message' => 'Your custom message here',
            //'allowEmpty' => false,
            //'required' => false,
            //'last' => false, // Stop validation after this rule
            //'on' => 'create', // Limit validation to 'create' or 'update' operations
            ),
        ),
    );

    //The Associations below have been created with all possible keys, those that are not needed can be removed

    /**
     * belongsTo associations
     *
     * @var array
     */
    public $belongsTo = array(
        'Courrier' => array(
            'className' => 'Courrier',
            'foreignKey' => 'courrier_id',
            'conditions' => '',
            'fields' => '',
            'order' => ''
        )
    );

    /**
     * hasMany
     *
     * @var mixed
     * @access public
     */
    public $hasMany = array(
        'Sendmail' => array(
            'className' => 'Sendmail',
            'foreignKey' => 'document_id',
            'dependent' => false,
            'conditions' => null,
            'fields' => null,
            'order' => null,
            'limit' => null,
            'offset' => null,
            'exclusive' => null,
            'finderQuery' => null,
            'counterQuery' => null
        )
    );

    /**
     *
     * @param type $id
     * @return type
     * @throws NotFoundException
     */
    public function generatePreview($id = null, $path = null) {
        if (!empty($id)) {
            $this->id = $id;
        }

        if (empty($this->id)) {
            throw new NotFoundException();
        }

        $pdfContent = '';
        $fileContent = '';
        $this->recursive = -1;
        $document = $this->read();
        $return = __d('document', 'Document.preview.error');
        if (!empty($path)) {
            require_once 'XML/RPC2/Client.php';
//            require_once 'XML/RPC.php';
            /*$content = base64_encode(file_get_contents($path));
            $params = array(new XML_RPC_Value($content, 'string'),
                new XML_RPC_Value('odt', 'string'),
                new XML_RPC_Value('pdf', 'string'),
                new XML_RPC_Value(false, 'boolean'),
                new XML_RPC_Value(true, 'boolean'));

            $url = Configure::read('CLOUDOOO_HOST') . ":" . Configure::read('CLOUDOOO_PORT');
            $msg = new XML_RPC_Message('convertFile', $params);
            $cli = new XML_RPC_Client('/', $url);
            $resp = $cli->send($msg);
            $pdfContent = base64_decode($resp->xv->me['string']); */
            
            $content = file_get_contents($path);
            $options = array(
                'uglyStructHack' => true
            );
            $url = 'http://' . Configure::read('CLOUDOOO_HOST') . ':' . Configure::read('CLOUDOOO_PORT');
            $cli = XML_RPC2_Client::create($url, $options);

            $result = $cli->convertFile(base64_encode($content), 'odt', 'pdf', false, true);
            $pdfContent = base64_decode($result);
            
        } else {
            //Conversion en pdf via Gedooo
            if (in_array($document['Document']['mime'], $this->convertibleMimeTypes)) {
//                require_once 'XML/RPC.php';
                require_once 'XML/RPC2/Client.php';
                //$content = $document['Document']['content'];

//                $extension = preg_replace('/^(txt|odt|pdf).*/', 'odt', $document['Document']['ext']);

                /*$params = array(new XML_RPC_Value($content, 'string'),
                    new XML_RPC_Value($extension, 'string'),
                    new XML_RPC_Value('pdf', 'string'),
                    new XML_RPC_Value(false, 'boolean'),
                    new XML_RPC_Value(true, 'boolean'));

                $url = Configure::read('CLOUDOOO_HOST') . ":" . Configure::read('CLOUDOOO_PORT');
                

//                debug($params);
//                die();

                $msg = new XML_RPC_Message('convertFile', $params);
                
                $cli = new XML_RPC_Client('/', $url);
                $resp = $cli->send($msg);                 
                $pdfContent = base64_decode($resp->xv->me['string']);                 */
//echo '<pre>';var_dump($document);die();
                $content = $document['Document']['content'];
                $options = array(
                    'uglyStructHack' => true
                );
                $url = 'http://' . Configure::read('CLOUDOOO_HOST') . ':' . Configure::read('CLOUDOOO_PORT');
                $cli = XML_RPC2_Client::create($url, $options);

                $result = $cli->convertFile(base64_encode($content), 'odt', 'pdf', false, true);
                $pdfContent = base64_decode($result);
                

                
            } else if ($document['Document']['mime'] == 'application/pdf' || $document['Document']['mime'] == 'PDF') {

                $pdfContent = $this->field('Document.content', array('Document.id' => $id));
            } else if ($document['Document']['mime'] == 'image/png' || $document['Document']['mime'] == 'image/jpeg') {
                // Si c'est une image, on retourne le contenu directement
                $return = $this->field('Document.content', array('Document.id' => $id));
            } else {
//                debug($document);
                require_once 'XML/RPC2/Client.php';
                //$content = base64_encode($document['Document']['content']);

                //$extension = preg_replace('/^(txt|odt|pdf).*/', 'odt', $document['Document']['ext']);
                /*$params = array(new XML_RPC_Value($content, 'string'),
                    new XML_RPC_Value($extension, 'string'),
                    new XML_RPC_Value('pdf', 'string'),
                    new XML_RPC_Value(false, 'boolean'),
                    new XML_RPC_Value(true, 'boolean'));

                $url = Configure::read('CLOUDOOO_HOST') . ":" . Configure::read('CLOUDOOO_PORT');

                $msg = new XML_RPC_Message('convertFile', $params);
                $cli = new XML_RPC_Client('/', $url);
                $resp = $cli->send($msg);                 
                if (!is_null($resp->xv)) {
                    $pdfContent = base64_decode($resp->xv->me['string']);
                } else {
//                    debug($document);
                }
                 */
                
                $content = $document['Document']['content'];
                $options = array(
                    'uglyStructHack' => true
                );
                $url = 'http://' . Configure::read('CLOUDOOO_HOST') . ':' . Configure::read('CLOUDOOO_PORT');
                $cli = XML_RPC2_Client::create($url, $options);
                $result = $cli->convertFile(base64_encode($content), 'odt', 'pdf', false, true);
                $pdfContent = base64_decode($result);
                
            }
        }

        //Conversion en flash
        if (!empty($pdfContent)) {
            $return = $pdfContent;
        }

        return $return;
    }

    /**
     * Récupère toutes les pièces jointes (associées à 1 flux) ayant la propriété main_doc à false
     * @param null|int|array $flux_id
     * @return mixed tableau d'annexes
     */
    public function getPiecesJointes($flux_id = null) {
        $conditions = array('main_doc' => false);

        if (!empty($flux_id))
            $conditions['courrier_id'] = $flux_id;

        $piecesjointes = $this->find('all', array(
            'conditions' => $conditions,
            'order' => array('id' => 'ASC'),
            'recursive' => -1
        ));

        return $piecesjointes;
    }

    /**
     * Génération auto de la référence unique du courrier
     * @param type $id
     * @return string|null
     */
    public function genNameUnique($id = null, $data = null) {
        $filename = null;
        $sql = "SELECT nextval('documents_name_seq'); /*" . mktime(true) . " " . $this->_lastSequenceDocument . "*/";


        $numSeqDoc = $this->query($sql);
        if ($numSeqDoc === false) {
            return null; // FIXME: Throw exception
        }
        $this->_lastSequenceDocument = $numSeqDoc[0][0]['nextval'];
        $filename = "fichier_" . sprintf("%012s", $numSeqDoc[0][0]['nextval']);
        return $filename;
    }

}
