<?php

App::uses('Aborganisme', 'Addressbook.Model');

/**
 * Contact model class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 7
 * @author Stéphane Sampaio
 * @copyright Initié par ADULLACT - Développé par ADULLACT Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		app.Model
 */
class Organisme extends Aborganisme {

    /**
     *
     * @var type
     */
    public $name = 'Organisme';

    /**
     *
     * @var type
     */
    public $useTable = 'organismes';
    public $hasMany = array(
        'Contact' => array(
            'className' => 'Contact',
            'foreignKey' => 'organisme_id',
            'dependent' => false,
            'conditions' => null,
            'fields' => null,
            'order' => null,
            'limit' => null,
            'offset' => null,
            'exclusive' => null,
            'finderQuery' => null,
            'counterQuery' => null
        ),
        'Courrier' => array(
            'className' => 'Courrier',
            'foreignKey' => 'organisme_id',
            'dependent' => false,
            'conditions' => null,
            'fields' => null,
            'order' => null,
            'limit' => null,
            'offset' => null,
            'exclusive' => null,
            'finderQuery' => null,
            'counterQuery' => null
        ),
        'Ordreservice' => array(
            'className' => 'Ordreservice',
            'foreignKey' => 'organisme_id',
            'dependent' => false,
            'conditions' => null,
            'fields' => null,
            'order' => null,
            'limit' => null,
            'offset' => null,
            'exclusive' => null,
            'finderQuery' => null,
            'counterQuery' => null
        )
    );

    /**
     *
     * @var type
     */
    public $belongsTo = array(
        'Addressbook' => array(
            'className' => 'Addressbook',
            'foreignKey' => 'addressbook_id',
            'conditions' => '',
            'fields' => '',
            'order' => ''
        ),
        'Ban' => array(
            'className' => 'Ban',
            'foreignKey' => 'ban_id',
            'conditions' => '',
            'fields' => '',
            'order' => ''
        ),
        'Bancommune' => array(
            'className' => 'Bancommune',
            'foreignKey' => 'bancommune_id',
            'conditions' => '',
            'fields' => '',
            'order' => ''
        )
    );

    /**
     *
     * @var type
     */
    public $hasAndBelongsToMany = array(
        'Recherche' => array(
            'className' => 'Recherche',
            'joinTable' => 'recherches_organismes',
            'foreignKey' => 'organisme_id',
            'associationForeignKey' => 'recherche_id',
            'unique' => null,
            'conditions' => null,
            'fields' => null,
            'order' => null,
            'limit' => null,
            'offset' => null,
            'finderQuery' => null,
            'deleteQuery' => null,
            'insertQuery' => null,
            'with' => 'RechercheOrganisme'
        ),
        'Operation' => array(
            'className' => 'Operation',
            'joinTable' => 'organismes_operations',
            'foreignKey' => 'organisme_id',
            'associationForeignKey' => 'operation_id',
            'unique' => null,
            'conditions' => null,
            'fields' => null,
            'order' => null,
            'limit' => null,
            'offset' => null,
            'finderQuery' => null,
            'deleteQuery' => null,
            'insertQuery' => null,
            'with' => 'OrganismeOperation'
        ),
        'Activite' => array(
            'className' => 'Activite',
            'joinTable' => 'activites_organismes',
            'foreignKey' => 'organisme_id',
            'associationForeignKey' => 'activite_id',
            'unique' => null,
            'conditions' => null,
            'fields' => null,
            'order' => null,
            'limit' => null,
            'offset' => null,
            'finderQuery' => null,
            'deleteQuery' => null,
            'insertQuery' => null,
            'with' => 'ActiviteOrganisme'
        )
    );

    /**
     *
     * @param type $created
     * @return type
     */
    public function beforeSave($options = array()) {
        parent::beforeSave($options);
        if (!empty($this->data['Organisme']['name'])) {
            $this->data['Organisme']['slug'] = strtolower(Inflector::slug($this->data['Organisme']['name']));
        }
        return true;
    }

    /**
     * Fonciton permettant de rechercher les organismes selon certains critères
     *
     * @param type $criteres
     * @return array()
     */
    public function search($criteres, $orgId = null) {

        /// Conditions de base
        if (!empty($orgId)) {
            $conditions = array(
                'Organisme.id' => $orgId
            );
        }
        // Critères sur l'utilisateur
        foreach (array('name', 'email') as $critereOrganisme) {
            if (isset($criteres['Organisme'][$critereOrganisme]) && !empty($criteres['Organisme'][$critereOrganisme])) {
                $conditions[] = 'Organisme.' . $critereOrganisme . ' ILIKE \'' . $this->wildcard('%' . $criteres['Organisme'][$critereOrganisme] . '%') . '\'';
            }
        }


        if (isset($criteres['Organisme']['active'])) {
            $conditions[] = array(
                'Organisme.active' => $criteres['Organisme']['active']
            );
        }

        if (Configure::read('Conf.SAERP')) {
            // Filtre pour les OPs
            if (!empty($criteres['Organisme']['Operation']['id'])) {
                    $opsids = $criteres['Organisme']['Operation'];
                    $organismeOps = $this->OrganismeOperation->find(
                        'all',
                        array(
                            'conditions' => array(
                                'OrganismeOperation.operation_id' => $opsids
                            ),
                            'contain' => false
                        )
                    );
                    $organismeOpsIds = Hash::extract($organismeOps, '{n}.OrganismeOperation.organisme_id');
                    $conditions[] = array(
                        'Organisme.id' => $organismeOpsIds
                    );
            }
            // Filtre pour les activités
            if (!empty($criteres['Organisme']['Activite']['id'])) {
                    $activitesids = $criteres['Organisme']['Activite'];
                    $organismeActivites = $this->ActiviteOrganisme->find(
                        'all',
                        array(
                            'conditions' => array(
                                'ActiviteOrganisme.activite_id' => $activitesids
                            ),
                            'contain' => false
                        )
                    );
                    $organismeActivitesIds = Hash::extract($organismeActivites, '{n}.ActiviteOrganisme.organisme_id');
                    $conditions[] = array(
                        'Organisme.id' => $organismeActivitesIds
                    );
            }

        }
        
        if (Configure::read('Conf.SAERP')) {
            $contain = array(
                'Operation',
                'Activite',
                'Contact' => array(
                    'conditions' => array(
                        'Contact.active' => true
                    ),
                    'order' => array(
                        'Contact.name ASC'
                    ),
                    'Event',
                    'Fonction',
                    'Operation'
                )
            );
        }
        else {
            $contain = array(
                'Contact' => array(
                    'conditions' => array(
                        'Contact.active' => true
                    ),
                    'order' => array(
                        'Contact.name ASC'
                    )
                )
            );
        }

        $query = array(
            'contain' => $contain,
            'order' => 'Organisme.name ASC',
            'conditions' => array(
                $conditions
            ),
        );

//        $query = array(
//            'contain' => array(
//                'Contact' => array(
//                    'conditions' => array(
//                        'Contact.active' => true
//                    ),
//                    'order' => array(
//                        'Contact.name ASC'
//                    )
//                /* ,
//                  'Contactinfo' => array(
//                  'conditions' => array(
//                  'Contactinfo.active' => true
//                  ),
//                  'order' => array(
//                  'Contactinfo.name ASC'
//                  )
//                  ) */
//                )
//            ),
////            'limit' => 20,
//            'order' => 'Organisme.name ASC',
//            'conditions' => array(
//                $conditions
//            ),
//        );

        return $query;
    }

    /**
     *
     * @param type $nom
     * @param type $prenom
     * @return type
     */
    public function getHomonyms($name) {
        $return = array();
        if (!empty($name)) {
//			$slug = strtolower($name);
            $name = strtolower(Inflector::slug($name));
//debug($name);
            $qd = array(
                'recursive' => -1,
                'fields' => array(
                    'Organisme.name',
                    'Addressbook.name'
                ),
                'contain' => false,
                'joins' => array(
                    $this->join($this->Addressbook->alias)
                ),
                'conditions' => array(
                    'OR' => array(
                        'Organisme.name ILIKE' => '%' . $name . '%',
                        "levenshtein('" . $name . "', Organisme.name) <= " . Configure::read('levenshteinThreshold')
                    )
                ),
                'order' => array(
                    "levenshtein('" . $name . "', Organisme.name) DESC",
                    'Organisme.name'
                )
            );
//debug($qd);
            $return = $this->find('all', $qd);
//debug($return);
        }
        return $return;
    }

    public function getNombreOrganismeByCarnetId($addressbook_id) {
        $qd = array(
            'conditions' => array(
                'Organisme.addressbook_id' => $addressbook_id
            )
        );

        $return = $this->find('count', $qd);
        return $return;
    }

}

?>