var gulp = require('gulp'),
        concat = require('gulp-concat'),
        uglify = require('gulp-uglify'),
        sass = require('gulp-sass'),
        contatCss = require('gulp-concat-css'),
        cleanCss = require('gulp-clean-css'),
        notify = require("gulp-notify"),
        bower = require('gulp-bower'),
        sassLint = require('gulp-sass-lint'),
        jshint = require('gulp-jshint'),
        stylish = require('jshint-stylish'),
        scssLintStylish = require('gulp-scss-lint-stylish');

var config = {
    jsPathSimple: 'app/webroot/source/js/',
    jsPath: 'app/webroot/source/js/*.js',
    sassPath: 'app/webroot/source/sass',
    bowerDir: 'app/webroot/source/component',
    publicPath: 'app/webroot'
};

//executer 'bower install' automatiquement
gulp.task('bower', function () {
    return bower();
});

// JSHint
// detect errors and potential
// problems in your JavaScript code.
gulp.task('lint', function () {
    return gulp.src(config.jsPath)
            .pipe(jshint())
            .pipe(jshint.reporter(stylish));
});

//Deplace tous les ficher js de projet du webroot/source/js au webroot/js
gulp.task('movejs', ['lint'], function () {
    return gulp.src(config.jsPath)
            .pipe(gulp.dest(config.publicPath + '/js'))
});

//UglifyJS is a JavaScript parser, minifier, compressor or beautifier toolkit.
gulp.task('contat', ['movejs'], function () {
    return gulp.src(config.jsPath)
            .pipe(concat('webgfc.js'))
            .pipe(uglify())
            .pipe(gulp.dest(config.publicPath + '/js'))
});

//A Node-only Sass linter for both sass and scss syntax!
gulp.task('sassLint', function () {
    return gulp.src(config.sassPath + '/src/webgfc.scss')
            .pipe(sassLint({customReport: scssLintStylish}))
            .pipe(sassLint.format())
            .pipe(sassLint.failOnError())
})

//composer et déplacer fichier webgfc.css
gulp.task('sass', ['bower', 'sassLint'], function () {
    return gulp.src([config.sassPath + '/src/webgfc.scss'])
            .pipe(sass())
            .pipe(gulp.dest(config.publicPath + '/css'))
});

var vendorCss = [
    config.bowerDir + '/font-awesome/scss/font-awesome.scss',
    config.bowerDir + '/select2/src/scss/core.scss',
    config.bowerDir + '/bootstrap-datepicker/dist/css/bootstrap-datepicker3.css',
    config.bowerDir + '/bootstrap-table/src/bootstrap-table.css',
    config.bowerDir + '/select2-bootstrap-theme/dist/select2-bootstrap.css',
    config.bowerDir + '/sweetalert2/dist/sweetalert2.css',
    config.bowerDir + '/bootstrap-treeview/dist/bootstrap-treeview.min.css'
]

//combiner tous les fichier *.css(qui vient de tous les plugin) dans un seul fichier vendor.css
gulp.task('contatCss', ['bower'], function () {
    return gulp.src(vendorCss)
            .pipe(sass())
            .pipe(contatCss("vendor.css"))
            .pipe(gulp.dest(config.publicPath + '/css'))
});

var vendorjs = [
    config.bowerDir + '/jquery/dist/jquery.js',
    config.bowerDir + '/jquery-ui/ui/jquery-ui.js',
    config.bowerDir + '/bootstrap-validator/dist/validator.js',
    config.bowerDir + '/bootstrap-sass/assets/javascripts/bootstrap.js',
    config.bowerDir + '/bootstrap-datepicker/dist/js/bootstrap-datepicker.js',
    config.bowerDir + '/bootstrap-table/src/bootstrap-table.js',
    config.bowerDir + '/select2/dist/js/select2.js',
    config.bowerDir + '/layer/src/layer.js',
    config.bowerDir + '/sweetalert2/dist/sweetalert2.js',
    config.bowerDir + '/bootstrap-treeview/dist/bootstrap-treeview.min.js',
    config.bowerDir + '/chart.js/dist/Chart.js',
    config.bowerDir + '/randomColor/randomColor.js',
    config.bowerDir + '/jquery.caret/dist/jquery.caret-1.5.0.js'
]

//combiner tous les fichier *.js(qui vient de tous les plugin) dans un seul fichier vendor.js
gulp.task('vendor', ['bower'], function () {
    return gulp.src(vendorjs)
            .pipe(concat('vendor.js'))
            .pipe(gulp.dest(config.publicPath + '/js'))
});

//deplacer les fichier de fonts dans un dossier
gulp.task('icon', ['bower'], function () {
    return gulp.src([config.bowerDir + '/font-awesome/fonts/**.*', config.bowerDir + '/bootstrap-sass/assets/fonts/**/**.*'])
            .pipe(gulp.dest(config.publicPath + '/fonts'))
});

//deplacer les skins qui va utiliser par layer dans public
gulp.task('skin', ['bower'], function () {
    return gulp.src(config.bowerDir + '/layer/src/skin/**/**.*')
            .pipe(gulp.dest(config.publicPath + '/js/skin'))
});

//deplacer tous les fichier qu'on a besoin de plugin pdfjs dans public
gulp.task('pdfjs', ['bower'], function () {
    return gulp.src(config.bowerDir + '/pdfjs-dist/**/*.*')
            .pipe(gulp.dest(config.publicPath + '/js/pdfjs'))
});

//deplacer le plugin de statistique dans un dossier public
gulp.task('gmaps', ['bower'], function () {
    return gulp.src([config.bowerDir + '/gmaps/gmaps.min.js'])
            .pipe(gulp.dest(config.publicPath + '/js/gmaps'))
})

//surveiller les changement de fichier *.scss et fichier *.js
gulp.task('watch', function () {
    gulp.watch(config.sassPath + '/**/*.scss', ['sass'])
    gulp.watch(config.jsPath, ['contat'])
});

gulp.task('default', ['contat', 'gmaps', 'contatCss', 'vendor', 'sass', 'icon', 'skin', 'pdfjs', 'watch']);
